// ===========================================================================
//  Copyright 1994 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       Checks
//  TYPE:       c++ header
//  PROJECT:    VEGA Base Library
//  CONTENT:    check macros
//  VERSION:    1.0.0
// ===========================================================================
//  AUTHORS:    rft     Robert F. Tobler
// ===========================================================================
//  HISTORY:
//
//  30-sep-94 13:40:13  rft     last modification
//  08-mar-94 18:31:43  rft     created
// ===========================================================================
//  MANUAL:
//
//  NAME
//      BaseChecks:
//
//  DESCRIPTION
//      This module defines the CH_CODE macro if the WITH_CHECKS symbol is
//      defined.
//
//      If WITH_CHECKS is defined, then the following definition is in
//      effect:
//
//              #define CH_CODE(code)        code
//
//      otherwise the macro expands to nothing:
//
//              #define CH_CODE(code)
//
//      This is useful to selectively compile consistency checks into the
//      code, without the need to eliminate them from the source code for
//      compiling the final version without checks.
//
//  EXAMPLE
//      If you want to selectively compile range checking into your code,
//      you might use a piece of code that resembles the following:
//
//              CH_CODE(
//                  if (index < 0 || index > maxIndex)
//                      EH_FATAL(Range_errors,"index == " << index);
//              )
//
//  SEE ALSO
//      VEGA, BaseLibrary
// ===========================================================================
#ifndef _BASE_CHECKS_H_
#define _BASE_CHECKS_H_

#include "VEGA/Base/defines.h"

#ifdef WITH_CHECKS
#define CH_CODE(code)             code
#else
#define CH_CODE(code)
#endif

#endif // _BASE_CHECKS_H_
// ===========================================================================
