// ===========================================================================
//  Copyright 1994 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       Tracer
//  TYPE:       c++ inline
//  PROJECT:    VEGA Base Library
//  CONTENT:    tracers which vanish if WITH_TRACERS isn't defined
//  VERSION:    1.0.0
// ===========================================================================
//  AUTHORS:    rft     Robert F. Tobler
// ===========================================================================
//  HISTORY:
//
//  30-sep-94 13:40:13  rft     last modification
//  15-Jul-93           rft     created
// ===========================================================================

extern "C++"
{

INLINE_MEMBER Tracer::Tracer(
	const char * newName,
	Tracer::State newState
	)
:
    parentTracer(0),
    name(newName),
    state(newState),
    counter(0)
{}

INLINE_MEMBER Tracer::Tracer(
	const char * newName,
	Tracer::State newState,
	Tracer & newParentTracer
	)
:
    parentTracer(&newParentTracer),
    name(newName),
    state(newState)
{}

INLINE_MEMBER void Tracer::turnOn()
{
    state = on;
}

INLINE_MEMBER void Tracer::turnOff()
{
    state = off;
}

INLINE_MEMBER int Tracer::isOff()
{
    return (state == off);
}

INLINE_GLOBAL ostream & operator<<(ostream & s,const Tracer & tracer)
{
    s << tracer.name;
    return s;
}

INLINE_MEMBER long Tracer::getAndIncCounter()
{
    return counter++;
}

INLINE_MEMBER void Tracer::enableLocalStream(ostream & s)
{
    localStream = &s;
}

INLINE_MEMBER void Tracer::disableLocalStream()
{
    localStream = 0;
}

INLINE_MEMBER void Tracer::setCommonStream(ostream & s)
{
    commonStream = &s;
}

} // extern "C++"

// ===========================================================================
