// ===========================================================================
//  Copyright 1994 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       testSystem
//  TYPE:       c++ code
//  PROJECT:    VEGA Base Library
//  CONTENT:    test the System module
//  VERSION:    1.0.0
// ===========================================================================
//  AUTHORS:    rft     Robert F. Tobler
// ===========================================================================
//  HISTORY:
//
//  30-sep-94 13:43:08  rft     last modification
//  30-sep-94 13:43:08  rft     created
// ===========================================================================

#include <VEGA/Base/test.h>

#include "tsystem.h"

#ifdef SGI
#include "tsystem.ipp"
#endif

static void testSizes(TestModule & module)
{
    Int8 int8max = INT8_MAX;

    module.test("Int8:bitSize",bitSize(Int8(0)) == 8);
    module.test("Int8:signBit",signBit(Int8(0)) == 7);
    module.test("Int8:INT8_MAX", (Int8)(int8max + 1) == INT8_MIN);

    Int16 int16max = INT16_MAX;

    module.test("Int16:bitSize",bitSize(Int16(0)) == 16);
    module.test("Int16:signBit",signBit(Int16(0)) == 15);
    module.test("Int16:INT16_MAX", (Int16)(int16max + 1) == INT16_MIN);

    Int32 int32max = INT32_MAX;		// BCC chokes on too large constants

    module.test("Int32:bitSize",bitSize(Int32(0)) == 32);
    module.test("Int32:signBit",signBit(Int32(0)) == 31);
    module.test("Int32:INT32_MAX", (Int32)(int32max + 1) == INT32_MIN);

    UInt8 uint8max = UINT8_MAX;

    module.test("UInt8:bitSize",bitSize(UInt8(0)) == 8);
    module.test("UInt8:signBit",signBit(UInt8(0)) == 8);
    module.test("UInt8:UINT8_MAX",(UInt8)(uint8max + 1) == 0);

    UInt16 uint16max = UINT16_MAX;

    module.test("UInt16:bitSize",bitSize(UInt16(0)) == 16);
    module.test("UInt16:signBit",signBit(UInt16(0)) == 16);
    module.test("UInt16:UINT16_MAX",(UInt16)(uint16max + 1) == 0);

    UInt32 uint32max = UINT32_MAX;

    module.test("UInt32:bitSize",bitSize(UInt32(0)) == 32);

    //
    // The following test can only be done if we have long longs.
    //
#ifdef LONGLONG
    module.test("UInt32:signBit",signBit(UInt32(0)) == 32);
#endif

    module.test("UInt32:UINT32_MAX",(UInt32)(uint32max + 1) == 0);
}

void testSystem(TestSuite & suite)
{
    TestModule module(suite, "System");

    testSizes(module);
}