// ===========================================================================
//  Copyright 1994 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       Color
//  TYPE:       c++ inline
//  PROJECT:    VEGA Graphics Library
//  CONTENT:    color classes
//  VERSION:    1.0.0
// ===========================================================================
//  AUTHORS:    rft     Robert F. Tobler
// ===========================================================================
//  HISTORY:
//
//  30-sep-94 14:08:16  rft     last modification
//  05-apr-94 14:30:43  rft     created
// ===========================================================================

extern "C++"
{

// ============================================================= class Color32

INLINE_MEMBER Color32::Color32(
	)
{
}

INLINE_MEMBER Color32::Color32(
	UInt8 red, UInt8 green, UInt8 blue, UInt8 alpha
	)
{
    c[0] = red;
    c[1] = green;
    c[2] = blue;
    c[3] = alpha;
}

INLINE_MEMBER float RGB::r() const { return c.x0(); }
INLINE_MEMBER float RGB::g() const { return c.x1(); }
INLINE_MEMBER float RGB::b() const { return c.x2(); }

INLINE_MEMBER Color32::Color32(const RGB & rgb)
{
    c[0] = UInt8(rgb.r() * 255.0);
    c[1] = UInt8(rgb.g() * 255.0);
    c[2] = UInt8(rgb.b() * 255.0);
    c[3] = 255;
}

INLINE_MEMBER double DRGB::r() const { return c.x0(); }
INLINE_MEMBER double DRGB::g() const { return c.x1(); }
INLINE_MEMBER double DRGB::b() const { return c.x2(); }

INLINE_MEMBER Color32::Color32(const DRGB & drgb)
{
    c[0] = UInt8(drgb.r() * 255.0);
    c[1] = UInt8(drgb.g() * 255.0);
    c[2] = UInt8(drgb.b() * 255.0);
    c[3] = 255;
}

INLINE_MEMBER float RGBA::r() const { return c.x0(); }
INLINE_MEMBER float RGBA::g() const { return c.x1(); }
INLINE_MEMBER float RGBA::b() const { return c.x2(); }
INLINE_MEMBER float RGBA::a() const { return c.x3(); }

INLINE_MEMBER Color32::Color32(const RGBA & rgba)
{
    c[0] = UInt8(rgba.r() * 255.0);
    c[1] = UInt8(rgba.g() * 255.0);
    c[2] = UInt8(rgba.b() * 255.0);
    c[3] = UInt8(rgba.a() * 255.0);
}

INLINE_MEMBER UInt8 Color32::r() const { return c[0]; }
INLINE_MEMBER UInt8 Color32::g() const { return c[1]; }
INLINE_MEMBER UInt8 Color32::b() const { return c[2]; }
INLINE_MEMBER UInt8 Color32::a() const { return c[3]; }

INLINE_MEMBER void Color32::r(UInt8 rNew) { c[0] = rNew; }
INLINE_MEMBER void Color32::g(UInt8 gNew) { c[0] = gNew; }
INLINE_MEMBER void Color32::b(UInt8 bNew) { c[0] = bNew; }
INLINE_MEMBER void Color32::a(UInt8 aNew) { c[0] = aNew; }

INLINE_MEMBER UInt8 & Color32::operator[](int index)
{
    return c[index];
}

INLINE_MEMBER UInt8 Color32::operator[](int index) const
{
    return c[index];
}

INLINE_GLOBAL int operator==(const Color32 & col1, const Color32 & col2)
{
    return (col1.c[0] == col2.c[0] &&
	    col1.c[1] == col2.c[1] &&
	    col1.c[2] == col2.c[2] &&
	    col1.c[3] == col2.c[3]);
}

INLINE_GLOBAL int operator!=(const Color32 & col1, const Color32 & col2)
{
    return (col1.c[0] != col2.c[0] ||
	    col1.c[1] != col2.c[1] ||
	    col1.c[2] != col2.c[2] ||
	    col1.c[3] != col2.c[3]);
}


// ================================================================= class RGB

INLINE_MEMBER RGB::RGB(
	)
{
}

INLINE_MEMBER RGB::RGB(
	float rNew,
	float gNew,
	float bNew
	)
:
    c(clamp(rNew,float(0.0),float(1.0)),
      clamp(gNew,float(0.0),float(1.0)),
      clamp(bNew,float(0.0),float(1.0)))
{
}

INLINE_MEMBER RGB::RGB(
	const FCrd3 & crd
	)
:
    c(crd)
{
}

INLINE_MEMBER RGB::RGB(
	const Color32 & col
	)
:
    c(col.c[0] / 255.0,
      col.c[1] / 255.0,
      col.c[2] / 255.0)
{
}

INLINE_MEMBER RGB::RGB(
	const DRGB & drgb
	)
:
    c(drgb.c.x0(),drgb.c.x1(),drgb.c.x2())
{
}

INLINE_MEMBER RGB::RGB(
	const RGBA & rgba
	)
:
    c(rgba.c.x0(),rgba.c.x1(),rgba.c.x2())
{
}

INLINE_MEMBER void RGB::r(float rNew) { c.x0(rNew); }
INLINE_MEMBER void RGB::g(float gNew) { c.x1(gNew); }
INLINE_MEMBER void RGB::b(float bNew) { c.x2(bNew); }

INLINE_MEMBER float & RGB::operator[](int index)
{
    return c[index];
}

INLINE_MEMBER float RGB::operator[](int index) const
{
    return c[index];
}

INLINE_MEMBER RGB & RGB::operator+=(const RGB & rgb)
{
    c += rgb.c;
    return *this;
}

INLINE_MEMBER RGB & RGB::operator-=(const RGB & rgb)
{
    c -= rgb.c;
    return *this;
}

INLINE_MEMBER RGB & RGB::operator*=(float scalar)
{
    c *= scalar;
    return *this;
}

INLINE_GLOBAL RGB operator+(const RGB & rgb1, const RGB & rgb2)
{
    return RGB(rgb1.c + rgb2.c);
}

INLINE_GLOBAL RGB operator-(const RGB & rgb1, const RGB & rgb2)
{
    return RGB(rgb1.c - rgb2.c);
}

INLINE_GLOBAL RGB operator*(float scalar, const RGB & rgb)
{
    return RGB(scalar * rgb.c);
}

INLINE_GLOBAL RGB operator*(const RGB & rgb,  float scalar)
{
    return RGB(rgb.c * scalar);
}

// ================================================================= class DRGB

INLINE_MEMBER DRGB::DRGB(
	)
{
}

INLINE_MEMBER DRGB::DRGB(
	double rNew,
	double gNew,
	double bNew
	)
:
    c(clamp(rNew,double(0.0),double(1.0)),
      clamp(gNew,double(0.0),double(1.0)),
      clamp(bNew,double(0.0),double(1.0)))
{
}

INLINE_MEMBER DRGB::DRGB(
	const Crd3 & crd
	)
:
    c(crd)
{
}

INLINE_MEMBER DRGB::DRGB(
	const Color32 & col
	)
:
    c(col.c[0] / 255.0,
      col.c[1] / 255.0,
      col.c[2] / 255.0)
{
}

INLINE_MEMBER DRGB::DRGB(
	const RGB & rgb
	)
:
    c(rgb.c.x0(),rgb.c.x1(),rgb.c.x2())
{
}

INLINE_MEMBER DRGB::DRGB(
	const RGBA & rgba
	)
:
    c(rgba.c.x0(),rgba.c.x1(),rgba.c.x2())
{
}

INLINE_MEMBER void DRGB::r(double rNew) { c.x0(rNew); }
INLINE_MEMBER void DRGB::g(double gNew) { c.x1(gNew); }
INLINE_MEMBER void DRGB::b(double bNew) { c.x2(bNew); }

INLINE_MEMBER double & DRGB::operator[](int index)
{
    return c[index];
}

INLINE_MEMBER double DRGB::operator[](int index) const
{
    return c[index];
}

INLINE_MEMBER DRGB & DRGB::operator+=(const DRGB & drgb)
{
    c += drgb.c;
    return *this;
}

INLINE_MEMBER DRGB & DRGB::operator-=(const DRGB & drgb)
{
    c -= drgb.c;
    return *this;
}

INLINE_MEMBER DRGB & DRGB::operator*=(double scalar)
{
    c *= scalar;
    return *this;
}

INLINE_GLOBAL DRGB operator+(const DRGB & drgb1, const DRGB & drgb2)
{
    return DRGB(drgb1.c + drgb2.c);
}

INLINE_GLOBAL DRGB operator-(const DRGB & drgb1, const DRGB & drgb2)
{
    return DRGB(drgb1.c - drgb2.c);
}

INLINE_GLOBAL DRGB operator*(double scalar, const DRGB & drgb)
{
    return DRGB(scalar * drgb.c);
}

INLINE_GLOBAL DRGB operator*(const DRGB & drgb,  double scalar)
{
    return DRGB(drgb.c * scalar);
}

// ================================================================ class RGBA

INLINE_MEMBER RGBA::RGBA(
	)
{
}

INLINE_MEMBER RGBA::RGBA(
	float rNew,
	float gNew,
	float bNew,
	float aNew
	)
:
    c(clamp(rNew,float(0.0),float(1.0)),
      clamp(gNew,float(0.0),float(1.0)),
      clamp(bNew,float(0.0),float(1.0)),
      clamp(aNew,float(0.0),float(1.0)))
{
}

INLINE_MEMBER RGBA::RGBA(
	const FCrd4 & crd
	)
:
    c(crd)
{
}

INLINE_MEMBER RGBA::RGBA(
	const Color32 & col
	)
:
    c(col.c[0] / 255.0,
      col.c[1] / 255.0,
      col.c[2] / 255.0,
      col.c[3] / 255.0)
{
}

INLINE_MEMBER RGBA::RGBA(
	const RGB & rgb,
	float alpha
	)
:
    c(rgb.c.x0(), rgb.c.x1(), rgb.c.x2(), alpha)
{
}

INLINE_MEMBER RGBA::RGBA(
	const DRGB & drgb,
	double alpha
	)
:
    c(drgb.c.x0(), drgb.c.x1(), drgb.c.x2(), alpha)
{
}

INLINE_MEMBER void RGBA::r(float rNew) { c.x0(rNew); }
INLINE_MEMBER void RGBA::g(float gNew) { c.x1(gNew); }
INLINE_MEMBER void RGBA::b(float bNew) { c.x2(bNew); }
INLINE_MEMBER void RGBA::a(float aNew) { c.x3(aNew); }

INLINE_MEMBER float & RGBA::operator[](int index)
{
    return c[index];
}

INLINE_MEMBER float RGBA::operator[](int index) const
{
    return c[index];
}

INLINE_MEMBER RGBA & RGBA::operator+=(const RGBA & rgba)
{
    c += rgba.c;
    return *this;
}

INLINE_MEMBER RGBA & RGBA::operator-=(const RGBA & rgba)
{
    c -= rgba.c;
    return *this;
}

INLINE_MEMBER RGBA & RGBA::operator*=(float scalar)
{
    c *= scalar;
    return *this;
}

INLINE_GLOBAL RGBA operator+(const RGBA & rgba1, const RGBA & rgba2)
{
    return RGBA(rgba1.c + rgba2.c);
}

INLINE_GLOBAL RGBA operator-(const RGBA & rgba1, const RGBA & rgba2)
{
    return RGBA(rgba1.c - rgba2.c);
}

INLINE_GLOBAL RGBA operator*(float scalar, const RGBA & rgba)
{
    return RGBA(scalar * rgba.c);
}

INLINE_GLOBAL RGBA operator*(const RGBA & rgba,  float scalar)
{
    return RGBA(rgba.c * scalar);
}

} // extern "C++"

// ===========================================================================
