// ===========================================================================
//  Copyright 1994 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       FVec3D
//  TYPE:       c++ inline
//  PROJECT:    VEGA Graphics Library
//  CONTENT:    3D vectors, points, and transformations using floats
//  VERSION:    1.0.0
// ===========================================================================
//  AUTHORS:    rft     Robert F. Tobler
// ===========================================================================
//  HISTORY:
//
//  30-sep-94 14:08:16  rft     last modification
//  08-Aug-93           rft     created
// ===========================================================================

extern "C++"
{

// =============================================================== class FVec3D

// -------------------------------------------------------------- constructors

INLINE_MEMBER FVec3D::FVec3D()
{
    TR_FUN0(FVec3D_constructors,FVec3D);
}

INLINE_MEMBER FVec3D::FVec3D(float xNew, float yNew, float zNew)
:
    c(xNew,yNew,zNew)
{
    TR_FUN3(FVec3D_constructors,FVec3D,xNew,yNew,zNew);
}

INLINE_MEMBER FVec3D::FVec3D(const FCrd3 & crd)
:
    c(crd)
{
    TR_FUN1(FVec3D_constructors,FVec3D,crd);
}

INLINE_MEMBER FVec3D::FVec3D(const FVec3D & vec)
:
    c(vec.c)
{
    TR_FUN1(FVec3D_constructors,FVec3D,vec);
}

INLINE_MEMBER FVec3D::~FVec3D()
{
    TR_MESSAGE(FVec3D_constructors,"~" << *this);
}

// ---- methods --------------------------------------------------------------

INLINE_MEMBER float FVec3D::operator[](int index) const
{
    return c[index];
}

INLINE_MEMBER float & FVec3D::operator[](int index)
{
    return c[index];
}

INLINE_MEMBER FVec3D & FVec3D::operator+=(const FVec3D & vec)
{
    c += vec.c;
    return *this;
}

INLINE_MEMBER FVec3D & FVec3D::operator-=(const FVec3D & vec)
{
    c -= vec.c;
    return *this;
}

INLINE_MEMBER FVec3D & FVec3D::operator*=(float scalar)
{
    c *= scalar;
    return *this;
}

INLINE_MEMBER float FVec3D::x() const { return c.x0(); }
INLINE_MEMBER float FVec3D::y() const { return c.x1(); }
INLINE_MEMBER float FVec3D::z() const { return c.x2(); }

INLINE_MEMBER void FVec3D::x(float xNew) { c.x0(xNew); }
INLINE_MEMBER void FVec3D::y(float yNew) { c.x1(yNew); }
INLINE_MEMBER void FVec3D::z(float zNew) { c.x2(zNew); }

INLINE_MEMBER void FVec3D::normalize()
{
    c.normalize();
}

INLINE_MEMBER FCrd3 FVec3D::getFCrd3() const
{
    return c;
}

INLINE_MEMBER FCrd4 FVec3D::getFCrd4() const
{
    return FCrd4(c.x0(),c.x1(),c.x2(),0.0);
}

INLINE_MEMBER FPnt3D::FPnt3D(const FCrd3 & crd)
:
    c(crd)
{
    TR_FUN1(FVec3D_constructors,FPnt3D,crd);
}

INLINE_MEMBER FPnt3D FVec3D::getFPnt3D() const
{
    return FPnt3D(c);
}

INLINE_GLOBAL ostream & operator<<(ostream & s, const FVec3D & vec)
{
    s << "FVec3D(" << vec.x() << "," << vec.y() << "," << vec.z() << ")";
    return s;
}

INLINE_GLOBAL FVec3D operator-(const FVec3D & vec)
{
    return FVec3D(-vec.c);
}

INLINE_GLOBAL FVec3D operator+(const FVec3D & vec1, const FVec3D & vec2)
{
    return FVec3D(vec1.c + vec2.c);
}

INLINE_GLOBAL FPnt3D operator+(const FPnt3D & pnt, const FVec3D & vec)
{
    return FPnt3D(pnt.c + vec.c);
}

INLINE_GLOBAL FVec3D operator-(const FVec3D & vec1, const FVec3D & vec2)
{
    return FVec3D(vec1.c - vec2.c);
}

INLINE_GLOBAL FPnt3D operator-(const FPnt3D & pnt, const FVec3D & vec)
{
    return FPnt3D(pnt.c - vec.c);
}

INLINE_GLOBAL FVec3D operator*(const FVec3D & vec, float scalar)
{
    return FVec3D(vec.c * scalar);
}

INLINE_GLOBAL FVec3D operator*(float scalar, const FVec3D & vec)
{
    return FVec3D(scalar * vec.c);
}

INLINE_GLOBAL FCrd3 getFCrd3(const FVec3D & vec)
{
    return vec.c;
}

INLINE_GLOBAL FCrd4 getFCrd4(const FVec3D & vec)
{
    return FCrd4(vec.c.x0(), vec.c.x1(), vec.c.x2(),0.0);
}

INLINE_GLOBAL FPnt3D getFPnt3D(const FVec3D & vec)
{
    return FPnt3D(vec.c);
}

INLINE_GLOBAL float squaredLength(const FVec3D & vec)
{
    return squaredLength(vec.c);
}

INLINE_GLOBAL float length(const FVec3D & vec)
{
    return length(vec.c);
}

INLINE_GLOBAL float dotProduct(const FVec3D & vec1, const FVec3D & vec2)
{
    return dotProduct(vec1.c,vec2.c);
}

INLINE_GLOBAL FVec3D crossProduct(const FVec3D & vec1, const FVec3D & vec2)
{
    return FVec3D(crossProduct(vec1.c,vec2.c));
}

INLINE_GLOBAL FVec3D normalizedCopy(const FVec3D & vec)
{
    return FVec3D(normalizedCopy(vec.c));
}

INLINE_GLOBAL FVec3D cwReciprocalCopy(const FVec3D & vec)
{
    return FVec3D(cwReciprocalCopy(vec.c));
}

INLINE_GLOBAL FVec3D cwMin(const FVec3D & vec1, const FVec3D & vec2)
{
    return FVec3D(cwMin(vec1.c,vec2.c));
}

INLINE_GLOBAL FVec3D cwMax(const FVec3D & vec1, const FVec3D & vec2)
{
    return FVec3D(cwMax(vec1.c,vec2.c));
}

INLINE_MEMBER FVec3D & FVec3D::operator*=(const FScale3D & scale)
{
    c.scale(scale.c);
    return *this;
}

INLINE_MEMBER FVec3D & FVec3D::operator*=(const FRotX3D & rot)
{
    c.rot0(rot.cosAlpha,rot.sinAlpha);
    return *this;
}

INLINE_MEMBER FVec3D & FVec3D::operator*=(const FRotY3D & rot)
{
    c.rot1(rot.cosAlpha,rot.sinAlpha);
    return *this;
}

INLINE_MEMBER FVec3D & FVec3D::operator*=(const FRotZ3D & rot)
{
    c.rot2(rot.cosAlpha,rot.sinAlpha);
    return *this;
}

INLINE_MEMBER FVec3D & FVec3D::operator*=(const FShearXZ3D & shear)
{
    c.shear01(shear.c.x0(),shear.c.x1());
    return *this;
}

INLINE_MEMBER FVec3D & FVec3D::operator*=(const FShearZX3D & shear)
{
    c.shear20(shear.c.x0(),shear.c.x1());
    return *this;
}

INLINE_MEMBER FVec3D & FVec3D::operator*=(const FShearYZ3D & shear)
{
    c.shear12(shear.c.x0(),shear.c.x1());
    return *this;
}

INLINE_MEMBER FVec3D & FVec3D::operator*=(const FLin3D & lin)
{
    c *= lin.m;
    return *this;
}

INLINE_MEMBER FVec3D & FVec3D::operator*=(const FTrafo3D & trafo)
{
    c *= trafo.m;
    return *this;
}

INLINE_GLOBAL FVec3D operator*(const FVec3D & vec, const FScale3D & scale)
{
    return FVec3D(prCrdScale(vec.c,scale.c));
}

INLINE_GLOBAL FVec3D operator*(const FVec3D & vec, const FRotX3D & rot)
{
    return FVec3D(prCrdRot0(vec.c,rot.cosAlpha,rot.sinAlpha));
}

INLINE_GLOBAL FVec3D operator*(const FVec3D & vec, const FRotY3D & rot)
{
    return FVec3D(prCrdRot1(vec.c,rot.cosAlpha,rot.sinAlpha));
}

INLINE_GLOBAL FVec3D operator*(const FVec3D & vec, const FRotZ3D & rot)
{
    return FVec3D(prCrdRot2(vec.c,rot.cosAlpha,rot.sinAlpha));
}

INLINE_GLOBAL FVec3D operator*(const FVec3D & vec, const FShearXZ3D & shear)
{
    return FVec3D(prCrdShear01(vec.c,shear.c.x0(),shear.c.x1()));
}

INLINE_GLOBAL FVec3D operator*(const FVec3D & vec, const FShearZX3D & shear)
{
    return FVec3D(prCrdShear20(vec.c,shear.c.x0(),shear.c.x1()));
}

INLINE_GLOBAL FVec3D operator*(const FVec3D & vec, const FShearYZ3D & shear)
{
    return FVec3D(prCrdShear12(vec.c,shear.c.x0(),shear.c.x1()));
}

INLINE_GLOBAL FVec3D operator*(const FVec3D & vec, const FLin3D & lin)
{
    return FVec3D(vec.c * lin.m);
}

INLINE_GLOBAL FVec3D operator*(const FVec3D & vec, const FTrafo3D & trafo)
{
    return FVec3D(vec.c * trafo.m);
}

// =============================================================== class FPnt3D

// -------------------------------------------------------------- constructors

INLINE_MEMBER FPnt3D::FPnt3D()
{
    TR_FUN0(FVec3D_constructors,FPnt3D);
}

INLINE_MEMBER FPnt3D::FPnt3D(float xNew, float yNew, float zNew)
:
    c(xNew,yNew,zNew)
{
    TR_FUN3(FVec3D_constructors,FPnt3D,xNew,yNew,zNew);
}

INLINE_MEMBER FPnt3D::FPnt3D(const FPnt3D & pnt)
:
    c(pnt.c)
{
    TR_FUN1(FVec3D_constructors,FPnt3D,pnt);
}

INLINE_MEMBER FPnt3D::~FPnt3D()
{
    TR_MESSAGE(FVec3D_constructors,"~" << *this);
}

// ---------------------------------------------------------- member functions

INLINE_MEMBER float FPnt3D::operator[](int index) const
{
    return c[index];
}

INLINE_MEMBER float & FPnt3D::operator[](int index)
{
    return c[index];
}

INLINE_MEMBER FPnt3D & FPnt3D::operator+=(const FVec3D & vec)
{
    c += vec.c;
    return *this;
}

INLINE_MEMBER FPnt3D & FPnt3D::operator-=(const FVec3D & vec)
{
    c -= vec.c;
    return *this;
}

INLINE_MEMBER float FPnt3D::x() const { return c.x0(); }
INLINE_MEMBER float FPnt3D::y() const { return c.x1(); }
INLINE_MEMBER float FPnt3D::z() const { return c.x2(); }

INLINE_MEMBER void FPnt3D::x(float xNew) { c.x0(xNew); }
INLINE_MEMBER void FPnt3D::y(float yNew) { c.x1(yNew); }
INLINE_MEMBER void FPnt3D::z(float zNew) { c.x2(zNew); }

INLINE_MEMBER FCrd3 FPnt3D::getFCrd3() const
{
    return c;
}

INLINE_MEMBER FCrd4 FPnt3D::getFCrd4() const
{
    return FCrd4(c.x0(),c.x1(),c.x2(),0.0);
}

INLINE_MEMBER FVec3D FPnt3D::getFVec3D() const
{
    return FVec3D(c);
}

INLINE_GLOBAL ostream & operator<<(ostream & s, const FPnt3D & vec)
{
    s << "(" << vec.x() << "," << vec.y() << "," << vec.z() << ")";
    return s;
}

INLINE_GLOBAL FVec3D operator-(const FPnt3D & pnt1, const FPnt3D & pnt2)
{
    return FVec3D(pnt1.c - pnt2.c);
}

INLINE_GLOBAL FCrd3 getFCrd3(const FPnt3D & pnt)
{
    return pnt.c;
}

INLINE_GLOBAL FCrd4 getFCrd4(const FPnt3D & pnt)
{
    return FCrd4(pnt.c.x0(), pnt.c.x1(), pnt.c.x2(),0.0);
}

INLINE_GLOBAL FVec3D getFVec3D(const FPnt3D & pnt)
{
    return FVec3D(pnt.c);
}

INLINE_GLOBAL float squaredDistanceFromOrigin(const FPnt3D & pnt)
{
    return (squaredLength(pnt.c));
}

INLINE_GLOBAL float distanceFromOrigin(const FPnt3D & pnt)
{
    return (length(pnt.c));
}

INLINE_GLOBAL float squaredDistance(const FPnt3D & pnt1, const FPnt3D & pnt2)
{
    return (squaredLength(pnt1.c - pnt2.c));
}

INLINE_GLOBAL float distance(const FPnt3D & pnt1, const FPnt3D & pnt2)
{
    return (length(pnt1.c - pnt2.c));
}

INLINE_GLOBAL FPnt3D cwMin(const FPnt3D & pnt1, const FPnt3D & pnt2)
{
    return FPnt3D(cwMin(pnt1.c,pnt2.c));
}

INLINE_GLOBAL FPnt3D cwMax(const FPnt3D & pnt1, const FPnt3D & pnt2)
{
    return FPnt3D(cwMax(pnt1.c,pnt2.c));
}

INLINE_MEMBER FPnt3D & FPnt3D::operator*=(const FScale3D & scale)
{
    c.scale(scale.c);
    return *this;
}

INLINE_MEMBER FPnt3D & FPnt3D::operator*=(const FRotX3D & rot)
{
    c.rot0(rot.cosAlpha,rot.sinAlpha);
    return *this;
}

INLINE_MEMBER FPnt3D & FPnt3D::operator*=(const FRotY3D & rot)
{
    c.rot0(rot.cosAlpha,rot.sinAlpha);
    return *this;
}

INLINE_MEMBER FPnt3D & FPnt3D::operator*=(const FRotZ3D & rot)
{
    c.rot0(rot.cosAlpha,rot.sinAlpha);
    return *this;
}

INLINE_MEMBER FPnt3D & FPnt3D::operator*=(const FShearXZ3D & shear)
{
    c.shear01(shear.c.x0(),shear.c.x1());
    return *this;
}

INLINE_MEMBER FPnt3D & FPnt3D::operator*=(const FShearZX3D & shear)
{
    c.shear20(shear.c.x0(),shear.c.x1());
    return *this;
}

INLINE_MEMBER FPnt3D & FPnt3D::operator*=(const FShearYZ3D & shear)
{
    c.shear12(shear.c.x0(),shear.c.x1());
    return *this;
}

INLINE_MEMBER FPnt3D & FPnt3D::operator*=(const FTrans3D & trans)
{
    c += trans.c;
    return *this;
}

INLINE_MEMBER FPnt3D & FPnt3D::operator*=(const FLin3D & lin)
{
    c *= lin.m;
    return *this;
}

INLINE_MEMBER FPnt3D & FPnt3D::operator*=(const FTrafo3D & trafo)
{
    c *= trafo.m;
    c += trafo.c;
    return *this;
}

INLINE_GLOBAL FPnt3D operator*(const FPnt3D & pnt, const FScale3D & scale)
{
    return FPnt3D(prCrdScale(pnt.c,scale.c));
}

INLINE_GLOBAL FPnt3D operator*(const FPnt3D & pnt, const FRotX3D & rot)
{
    return FPnt3D(prCrdRot0(pnt.c,rot.cosAlpha,rot.sinAlpha));
}

INLINE_GLOBAL FPnt3D operator*(const FPnt3D & pnt, const FRotY3D & rot)
{
    return FPnt3D(prCrdRot1(pnt.c,rot.cosAlpha,rot.sinAlpha));
}

INLINE_GLOBAL FPnt3D operator*(const FPnt3D & pnt, const FRotZ3D & rot)
{
    return FPnt3D(prCrdRot2(pnt.c,rot.cosAlpha,rot.sinAlpha));
}

INLINE_GLOBAL FPnt3D operator*(const FPnt3D & pnt, const FShearXZ3D & shear)
{
    return FPnt3D(prCrdShear01(pnt.c,shear.c.x0(),shear.c.x1()));
}

INLINE_GLOBAL FPnt3D operator*(const FPnt3D & pnt, const FShearZX3D & shear)
{
    return FPnt3D(prCrdShear20(pnt.c,shear.c.x0(),shear.c.x1()));
}

INLINE_GLOBAL FPnt3D operator*(const FPnt3D & pnt, const FShearYZ3D & shear)
{
    return FPnt3D(prCrdShear12(pnt.c,shear.c.x0(),shear.c.x1()));
}

INLINE_GLOBAL FPnt3D operator*(const FPnt3D & pnt, const FTrans3D & trans)
{
    return FPnt3D(pnt.c + trans.c);
}

INLINE_GLOBAL FPnt3D operator*(const FPnt3D & pnt, const FLin3D & lin)
{
    return FPnt3D(pnt.c * lin.m);
}

INLINE_GLOBAL FPnt3D operator*(const FPnt3D & pnt, const FTrafo3D & trafo)
{
    return FPnt3D(pnt.c * trafo.m + trafo.c);
}

// ============================================================== class FScale3D

INLINE_MEMBER FScale3D::FScale3D(
	)
:
    c(1,1,1)
{
    TR_FUN0(FVec3D_constructors, FScale3D);
}

INLINE_MEMBER FScale3D::FScale3D(float scalar)
:
    c(scalar,scalar,scalar)
{
    TR_FUN1(FVec3D_constructors,FScale3D,scalar);
}

INLINE_MEMBER FScale3D::FScale3D(float x, float y, float z)
:
    c(x,y,z)
{
    TR_FUN3(FVec3D_constructors,FScale3D,x,y,z);
}

INLINE_MEMBER FScale3D::FScale3D(const FCrd3 & scale)
:
    c(scale)
{
    TR_FUN1(FVec3D_constructors,FScale3D,scale);
}

INLINE_MEMBER FScale3D::FScale3D(const FScale3D & scale)
:
    c(scale.c)
{
    TR_FUN1(FVec3D_constructors,FScale3D,scale);
}

INLINE_MEMBER FScale3D::~FScale3D()
{
    TR_MESSAGE(FVec3D_constructors,"~" << *this);
}

INLINE_MEMBER FScale3D & FScale3D::operator*=(const FScale3D & scale)
{
    c.cwMultiplyBy(scale.c);
    return *this;
}

INLINE_MEMBER void FScale3D::invert()
{
    if (c.x0() == 0.0 ||                // use explicit accessor function
        c.x1() == 0.0 ||                // to avoid range checks
        c.x2() == 0.0)
    {
        EH_ERROR(FVec3D_errors, ERROR_FVEC3D_SINGULAR_SCALE, "singular scale");
        return;
    }

    c.x0(1.0/c.x0());
    c.x1(1.0/c.x1());
    c.x2(1.0/c.x2());
}

INLINE_GLOBAL ostream & operator<<(ostream & s, const FScale3D & scale)
{
    s << "FScale3D(" << scale.c.x0() << "," << scale.c.x1() << ","
      << scale.c.x2() << ")";
    return s;
}

INLINE_GLOBAL FScale3D operator*(const FScale3D & scale1, const FScale3D & scale2)
{
    return FScale3D(cwProduct(scale1.c,scale2.c));
}


INLINE_GLOBAL FScale3D inverseCopy(const FScale3D & scale)
{
    if (scale.c.x0() == 0.0 ||
        scale.c.x1() == 0.0 ||
        scale.c.x2() == 0.0)
    {
        EH_ERROR(FVec3D_errors, ERROR_FVEC3D_SINGULAR_SCALE, "singular scale");
        return scale;
    }

    return FScale3D(1.0/scale.c.x0(), 1.0/scale.c.x1(), 1.0/scale.c.x2());
}


// =============================================================== class FRotX3D

INLINE_MEMBER FRotX3D::FRotX3D(
	)
{
    TR_FUN0(FVec3D_constructors,FRotX3D);
}

INLINE_MEMBER FRotX3D::FRotX3D(float newAlpha)
:
    alpha(newAlpha),
    cosAlpha(cos(newAlpha)),
    sinAlpha(sin(newAlpha))
{
    TR_FUN1(FVec3D_constructors,FRotX3D,newAlpha);
}

INLINE_MEMBER FRotX3D::FRotX3D(
	float newAlpha, float newCosAlpha, float newSinAlpha
	)
:
    alpha(newAlpha),
    cosAlpha(newCosAlpha),
    sinAlpha(newSinAlpha)
{
    TR_FUN3(FVec3D_constructors,FRotX3D,newAlpha,newCosAlpha,newSinAlpha);
}

INLINE_MEMBER FRotX3D::FRotX3D(const FRotX3D & rot)
:
    alpha(rot.alpha),
    cosAlpha(rot.cosAlpha),
    sinAlpha(rot.sinAlpha)
{
    TR_FUN1(FVec3D_constructors,FRotX3D,rot);
}

INLINE_MEMBER FRotX3D::~FRotX3D()
{
    TR_MESSAGE(FVec3D_constructors,"~" << *this);
}

INLINE_MEMBER FRotX3D & FRotX3D::operator*=(const FRotX3D & rot)
{
    alpha += rot.alpha;
    cosAlpha = cos(alpha);
    sinAlpha = sin(alpha);
    
    return *this;
}

INLINE_MEMBER float FRotX3D::getAlpha() const
{
    return alpha;
}

INLINE_MEMBER void FRotX3D::invert()
{
    alpha = -alpha;
    sinAlpha = -sinAlpha;
}

INLINE_GLOBAL ostream & operator<<(ostream & s, const FRotX3D & rot)
{
    s << "FRotX3D(" << rot.alpha << ")";
    return s;
}

INLINE_GLOBAL FRotX3D inverseCopy(const FRotX3D & rot)
{
    return FRotX3D(-rot.alpha,rot.cosAlpha,-rot.sinAlpha);
}

INLINE_GLOBAL FRotX3D operator*(const FRotX3D & rot1, const FRotX3D & rot2)
{
    return FRotX3D(rot1.alpha + rot2.alpha);
}

// =============================================================== class FRotY3D

INLINE_MEMBER FRotY3D::FRotY3D(
	)
{
    TR_FUN0(FVec3D_constructors,FRotY3D);
}

INLINE_MEMBER FRotY3D::FRotY3D(float newAlpha)
:
    alpha(newAlpha),
    cosAlpha(cos(newAlpha)),
    sinAlpha(sin(newAlpha))
{
    TR_FUN1(FVec3D_constructors,FRotY3D,newAlpha);
}

INLINE_MEMBER FRotY3D::FRotY3D(
	float newAlpha,float newCosAlpha, float newSinAlpha
	)
:
    alpha(newAlpha),
    cosAlpha(newCosAlpha),
    sinAlpha(newSinAlpha)
{
    TR_FUN3(FVec3D_constructors,FRotY3D,newAlpha,newCosAlpha,newSinAlpha);
}

INLINE_MEMBER FRotY3D::FRotY3D(const FRotY3D & rot)
:
    alpha(rot.alpha),
    cosAlpha(rot.cosAlpha),
    sinAlpha(rot.sinAlpha)
{
    TR_FUN1(FVec3D_constructors,FRotY3D,rot);
}

INLINE_MEMBER FRotY3D::~FRotY3D()
{
    TR_MESSAGE(FVec3D_constructors,"~" << *this);
}

INLINE_MEMBER FRotY3D & FRotY3D::operator*=(const FRotY3D & rot)
{
    alpha += rot.alpha;
    cosAlpha = cos(alpha);
    sinAlpha = sin(alpha);
    
    return *this;
}

INLINE_MEMBER float FRotY3D::getAlpha() const
{
    return alpha;
}

INLINE_MEMBER void FRotY3D::invert()
{
    alpha = -alpha;
    sinAlpha = -sinAlpha;
}

INLINE_GLOBAL ostream & operator<<(ostream & s, const FRotY3D & rot)
{
    s << "FRotY3D(" << rot.alpha << ")";
    return s;
}

INLINE_GLOBAL FRotY3D inverseCopy(const FRotY3D & rot)
{
    return FRotY3D(-rot.alpha,rot.cosAlpha,-rot.sinAlpha);
}

INLINE_GLOBAL FRotY3D operator*(const FRotY3D & rot1, const FRotY3D & rot2)
{
    return FRotY3D(rot1.alpha + rot2.alpha);
}

// =============================================================== class FRotZ3D

INLINE_MEMBER FRotZ3D::FRotZ3D(
	)
{
    TR_FUN0(FVec3D_constructors,FRotZ3D);
}

INLINE_MEMBER FRotZ3D::FRotZ3D(float newAlpha)
:
    alpha(newAlpha),
    cosAlpha(cos(newAlpha)),
    sinAlpha(sin(newAlpha))
{
    TR_FUN1(FVec3D_constructors,FRotY3D,newAlpha);
}

INLINE_MEMBER FRotZ3D::FRotZ3D(
	float newAlpha, float newCosAlpha, float newSinAlpha
	)
:
    alpha(newAlpha),
    cosAlpha(newCosAlpha),
    sinAlpha(newSinAlpha)
{
    TR_FUN3(FVec3D_constructors,FRotY3D,newAlpha,newCosAlpha,newSinAlpha);
}

INLINE_MEMBER FRotZ3D::FRotZ3D(const FRotZ3D & rot)
:
    alpha(rot.alpha),
    cosAlpha(rot.cosAlpha),
    sinAlpha(rot.sinAlpha)
{
    TR_FUN1(FVec3D_constructors,FRotY3D,rot);
}

INLINE_MEMBER FRotZ3D::~FRotZ3D()
{
    TR_MESSAGE(FVec3D_constructors,"~" << *this);
}

INLINE_MEMBER FRotZ3D & FRotZ3D::operator*=(const FRotZ3D & rot)
{
    alpha += rot.alpha;
    cosAlpha = cos(alpha);
    sinAlpha = sin(alpha);

    return *this;
}

INLINE_MEMBER float FRotZ3D::getAlpha() const
{
    return alpha;
}

INLINE_MEMBER void FRotZ3D::invert()
{
    alpha = -alpha;
    sinAlpha = -sinAlpha;
}

INLINE_GLOBAL ostream & operator<<(ostream & s, const FRotZ3D & rot)
{
    s << "FRotZ3D(" << rot.alpha << ")";
    return s;
}

INLINE_GLOBAL FRotZ3D inverseCopy(const FRotZ3D & rot)
{
    return FRotZ3D(-rot.alpha,rot.cosAlpha,-rot.sinAlpha);
}

INLINE_GLOBAL FRotZ3D operator*(const FRotZ3D & rot1, const FRotZ3D & rot2)
{
    return FRotZ3D(rot1.alpha + rot2.alpha);
}

// =============================================================== class FRot3D

INLINE_MEMBER FRot3D::FRot3D(
        )
{
    TR_FUN0(FVec3D_constructors,FRot3D);
}

INLINE_MEMBER FRot3D::FRot3D(
        const FVec3D & vec,
        float alpha
        )
{
    alpha *= 0.5;			// quaternions always use half-angles

    float s = sin(alpha) / length(vec);
    c.x0(vec.x() * s);
    c.x1(vec.y() * s);
    c.x2(vec.z() * s);
    c.x3(- cos(alpha));
    TR_FUN2(FVec3D_constructors,FRot3D,vec,alpha);
}

INLINE_MEMBER FRot3D::FRot3D(
	const FVec3D & vec
	)
:
    c(vec.getFCrd4())
{
    TR_FUN1(FVec3D_constructors,FRot3D,vec);
}

INLINE_MEMBER FRot3D::FRot3D(
    	float aNew,
	float bNew,
	float cNew,
	float dNew
        )
:
    c(aNew,bNew,cNew,dNew)
{
    TR_FUN4(FVec3D_constructors,FRot3D,aNew,bNew,cNew,dNew);
}

INLINE_MEMBER FRot3D::FRot3D(
        const FCrd4 & rot
        )
:
    c(rot)
{
    TR_FUN1(FVec3D_constructors,FRot3D,rot);
}

INLINE_MEMBER FRot3D::FRot3D(const FRotX3D & rot)
:
    c(sin(0.5 * rot.alpha), 0.0, 0.0, - cos(0.5 * rot.alpha))
{
    TR_FUN1(FVec3D_constructors,FRot3D,rot);
}

INLINE_MEMBER FRot3D::FRot3D(const FRotY3D & rot)
:
    c(0.0, sin(0.5 * rot.alpha), 0.0, - cos(0.5 * rot.alpha))
{
    TR_FUN1(FVec3D_constructors,FRot3D,rot);
}

INLINE_MEMBER FRot3D::FRot3D(const FRotZ3D & rot)
:
    c(0.0, 0.0, sin(0.5 * rot.alpha), - cos(0.5 * rot.alpha))
{
    TR_FUN1(FVec3D_constructors,FRot3D,rot);
}

INLINE_MEMBER FRot3D::FRot3D(const FRot3D & rot)
:
    c(rot.c)
{
    TR_FUN1(FVec3D_constructors,FRot3D,rot);
}

INLINE_MEMBER FRot3D::~FRot3D()
{
    TR_MESSAGE(FVec3D_constructors,"~" << *this);
}

INLINE_GLOBAL FRot3D operator*(const FRot3D & rot1, const FRot3D & rot2)
{
    return
        FRot3D
        (
	    FCrd4
	    (
		rot1.c.x0() * rot2.c.x3() + rot1.c.x3() * rot2.c.x0() +
		rot1.c.x1() * rot2.c.x2() - rot1.c.x2() * rot2.c.x1(),
    
		rot1.c.x1() * rot2.c.x3() + rot1.c.x3() * rot2.c.x1() +
		rot1.c.x2() * rot2.c.x0() - rot1.c.x0() * rot2.c.x2(),
    
		rot1.c.x2() * rot2.c.x3() + rot1.c.x3() * rot2.c.x2() +
		rot1.c.x0() * rot2.c.x1() - rot1.c.x1() * rot2.c.x0(),
		
		rot1.c.x3() * rot2.c.x3() - rot1.c.x2() * rot2.c.x2() -
		rot1.c.x1() * rot2.c.x1() - rot1.c.x0() * rot2.c.x0()
	    )
        );
}

INLINE_MEMBER FRot3D & FRot3D::operator*=(const FRot3D & rot)
{
    *this = *this * rot;
    return *this;
}

INLINE_MEMBER float FRot3D::getAlpha() const
{
    return(2 * acos(-c.x3()));
}

INLINE_MEMBER FVec3D FRot3D::getFVec3D() const
{
    return FVec3D(c.x0(),c.x1(),c.x2());
}

INLINE_MEMBER void FRot3D::invert()
{
    c.x3(-c.x3());
}

INLINE_GLOBAL ostream & operator<<(ostream & s, const FRot3D & rot)
{
    s << "FRot3D(" << rot.c.x0() << "," << rot.c.x1() << "," << rot.c.x2()
      << "," << rot.c.x3() << ")";
    return s;
}

INLINE_GLOBAL FRot3D inverseCopy(const FRot3D & rot)
{
    return FRot3D(FCrd4(rot.c.x0(),rot.c.x1(),rot.c.x2(),-rot.c.x3()));
}

// =========================================================== class FShearXZ3D

INLINE_MEMBER FShearXZ3D::FShearXZ3D(
        )
{
    TR_FUN0(FVec3D_constructors,FShearXZ3D);
}

INLINE_MEMBER FShearXZ3D::FShearXZ3D(const FShearXZ3D & shear)
:
    c(shear.c)
{
    TR_FUN1(FVec3D_constructors,FShearXZ3D,shear);
}

INLINE_MEMBER FShearXZ3D::FShearXZ3D(float xNew, float yNew)
:
    c(xNew,yNew)
{
    TR_FUN2(FVec3D_constructors,FShearXZ3D,xNew,yNew);
}

INLINE_MEMBER FShearXZ3D::FShearXZ3D(const Crd2 & crd)
:
    c(crd)
{
    TR_FUN1(FVec3D_constructors,FShearXZ3D,crd);
}

INLINE_MEMBER FShearXZ3D::~FShearXZ3D()
{
    TR_MESSAGE(FVec3D_constructors,"~" << *this);
}


INLINE_MEMBER FShearXZ3D & FShearXZ3D::operator*=(const FShearXZ3D & shear)
{
    c += shear.c;
    return *this;
}

INLINE_MEMBER void FShearXZ3D::invert()
{
    c = -c;
}

INLINE_GLOBAL ostream & operator<<(ostream & s, const FShearXZ3D & shear)
{
    s << "FShearXZ3D(" << shear.c.x0() << ',' << shear.c.x1() << ')';
    return s;
}

INLINE_GLOBAL FShearXZ3D operator*(const FShearXZ3D & sh1, const FShearXZ3D & sh2)
{
    return FShearXZ3D(sh1.c + sh2.c);
}

INLINE_GLOBAL FShearXZ3D inverseCopy(const FShearXZ3D & shear)
{
    return FShearXZ3D(-shear.c);
}

// =========================================================== class FShearZX3D

INLINE_MEMBER FShearZX3D::FShearZX3D(
        )
{
    TR_FUN0(FVec3D_constructors,FShearZX3D);
}

INLINE_MEMBER FShearZX3D::FShearZX3D(const FShearZX3D & shear)
:
    c(shear.c)
{
    TR_FUN1(FVec3D_constructors,FShearZX3D,shear);
}

INLINE_MEMBER FShearZX3D::FShearZX3D(float zNew, float xNew)
:
    c(zNew,xNew)
{
    TR_FUN2(FVec3D_constructors,FShearZX3D,zNew,xNew);
}

INLINE_MEMBER FShearZX3D::FShearZX3D(const Crd2 & crd)
:
    c(crd)
{
    TR_FUN1(FVec3D_constructors,FShearZX3D,crd);
}

INLINE_MEMBER FShearZX3D::~FShearZX3D()
{
    TR_MESSAGE(FVec3D_constructors,"~" << *this);
}


INLINE_MEMBER FShearZX3D & FShearZX3D::operator*=(const FShearZX3D & shear)
{
    c += shear.c;
    return *this;
}

INLINE_MEMBER void FShearZX3D::invert()
{
    c = -c;
}

INLINE_GLOBAL ostream & operator<<(ostream & s, const FShearZX3D & shear)
{
    s << "FShearZX3D(" << shear.c.x0() << ',' << shear.c.x1() << ')';
    return s;
}

INLINE_GLOBAL FShearZX3D operator*(const FShearZX3D & sh1, const FShearZX3D & sh2)
{
    return FShearZX3D(sh1.c + sh2.c);
}

INLINE_GLOBAL FShearZX3D inverseCopy(const FShearZX3D & shear)
{
    return FShearZX3D(-shear.c);
}

// =========================================================== class FShearYZ3D

INLINE_MEMBER FShearYZ3D::FShearYZ3D(
        )
{
    TR_FUN0(FVec3D_constructors,FShearYZ3D);
}

INLINE_MEMBER FShearYZ3D::FShearYZ3D(const FShearYZ3D & shear)
:
    c(shear.c)
{
    TR_FUN1(FVec3D_constructors,FShearYZ3D,shear);
}

INLINE_MEMBER FShearYZ3D::FShearYZ3D(float yNew, float zNew)
:
    c(yNew,zNew)
{
    TR_FUN2(FVec3D_constructors,FShearYZ3D,yNew,zNew);
}

INLINE_MEMBER FShearYZ3D::FShearYZ3D(const Crd2 & crd)
:
    c(crd)
{
    TR_FUN1(FVec3D_constructors,FShearYZ3D,crd);
}

INLINE_MEMBER FShearYZ3D::~FShearYZ3D()
{
    TR_MESSAGE(FVec3D_constructors,"~" << *this);
}


INLINE_MEMBER FShearYZ3D & FShearYZ3D::operator*=(const FShearYZ3D & shear)
{
    c += shear.c;
    return *this;
}

INLINE_MEMBER void FShearYZ3D::invert()
{
    c = -c;
}

INLINE_GLOBAL ostream & operator<<(ostream & s, const FShearYZ3D & shear)
{
    s << "FShearYZ3D(" << shear.c.x0() << ',' << shear.c.x1() << ')';
    return s;
}

INLINE_GLOBAL FShearYZ3D operator*(const FShearYZ3D & sh1, const FShearYZ3D & sh2)
{
    return FShearYZ3D(sh1.c + sh2.c);
}

INLINE_GLOBAL FShearYZ3D inverseCopy(const FShearYZ3D & shear)
{
    return FShearYZ3D(-shear.c);
}

// =============================================================== class FLin3D

INLINE_MEMBER FLin3D::FLin3D()
{
    TR_FUN0(FVec3D_constructors,FLin3D);
}

INLINE_MEMBER FLin3D::FLin3D(const FScale3D & scale)
:
    m(  scale.c.x0(),   0.0,            0.0,
        0.0,            scale.c.x1(),   0.0,
        0.0,            0.0,            scale.c.x2()    )
{
    TR_FUN1(FVec3D_constructors,FLin3D,scale);
}

INLINE_MEMBER FLin3D::FLin3D(const FRotX3D & rot)
:
    m(  1.0,      0.0,            0.0,
        0.0,    rot.cosAlpha,   rot.sinAlpha,
        0.0,      -rot.sinAlpha,  rot.cosAlpha    )
{
    TR_FUN1(FVec3D_constructors,FLin3D,rot);
}

INLINE_MEMBER FLin3D::FLin3D(const FRotY3D & rot)
:
    m(  rot.cosAlpha,   0.0,    -rot.sinAlpha,
        0.0,            1.0,    0.0,
        rot.sinAlpha,   0.0,    rot.cosAlpha    )
{
    TR_FUN1(FVec3D_constructors,FLin3D,rot);
}

INLINE_MEMBER FLin3D::FLin3D(const FShearXZ3D & shear)
:
    m(  1.0,		0.0,		0.0,
        0.0,            1.0,		0.0,
        shear.c.x0(),	shear.c.x1(),	1.0    )
{
    TR_FUN1(FVec3D_constructors,FLin3D,shear);
}

INLINE_MEMBER FLin3D::FLin3D(const FShearZX3D & shear)
:
    m(  1.0,		0.0,		0.0,
        shear.c.x1(),	1.0,   shear.c.x0(),
        0.0,		0.0,		1.0    )
{
    TR_FUN1(FVec3D_constructors,FLin3D,shear);
}

INLINE_MEMBER FLin3D::FLin3D(const FShearYZ3D & shear)
:
    m(  1.0,   shear.c.x0(),   shear.c.x1(),
        0.0,            1.0,		0.0,
        0.0,		0.0,		1.0    )
{
    TR_FUN1(FVec3D_constructors,FLin3D,shear);
}

INLINE_MEMBER FLin3D::FLin3D(const FRotZ3D & rot)
:
    m(  rot.cosAlpha,   rot.sinAlpha,   0.0,
        -rot.sinAlpha,  rot.cosAlpha,   0.0,
        0.0,            0.0,            1.0     )
{
    TR_FUN1(FVec3D_constructors,FLin3D,rot);
}

INLINE_MEMBER FLin3D::FLin3D(const FRot3D & rot)
{
    float aa = square(rot.c.x0());
    float ab2 = 2 * rot.c.x0() * rot.c.x1();
    float ac2 = 2 * rot.c.x0() * rot.c.x2();
    float ad2 = 2 * rot.c.x0() * rot.c.x3();
    float bb = square(rot.c.x1());
    float bc2 = 2 * rot.c.x1() * rot.c.x2();
    float bd2 = 2 * rot.c.x1() * rot.c.x3();
    float cc = square(rot.c.x2());
    float cd2 = 2 * rot.c.x2() * rot.c.x3();
    float dd = square(rot.c.x3());
    m = FMat3(aa - bb + dd - cc, ab2 - cd2, ac2 + bd2,
             ab2 + cd2, bb - aa + dd - cc, bc2 - ad2,
             ac2 - bd2, bc2 + ad2, cc - aa + dd - bb); 
    TR_FUN1(FVec3D_constructors,FLin3D,rot);
}

INLINE_MEMBER FLin3D::FLin3D(const FMat3 & mat)
:
    m(mat)
{
    TR_FUN1(FVec3D_constructors,FLin3D,mat);
}

INLINE_MEMBER FLin3D::FLin3D(const FLin3D & lin)
:
    m(lin.m)
{
    TR_FUN1(FVec3D_constructors,FLin3D,lin);
}

INLINE_MEMBER FLin3D::FLin3D(
	const FVec3D & vx, const FVec3D & vy, const FVec3D & vz
	)
:
    m(vx.c.x0(),vx.c.x1(),vx.c.x2(),
      vy.c.x0(),vy.c.x1(),vy.c.x2(),
      vz.c.x0(),vz.c.x1(),vz.c.x2())
{
    TR_FUN3(FVec3D_constructors,FLin3D,vx,vy,vz);
}

INLINE_MEMBER FLin3D::FLin3D(
        float xx, float xy, float xz,
        float yx, float yy, float yz,
        float zx, float zy, float zz
        )
:
    m(xx,xy,xz, yx,yy,yz, zx,zy,zz)
{
    TR_FUN4(FVec3D_constructors,FLin3D,xx,xy,xz,"...");
}

INLINE_MEMBER FLin3D::~FLin3D()
{
    TR_MESSAGE(FVec3D_constructors,"~" << *this);
}

INLINE_MEMBER FVec3D FLin3D::getVecX() const
{
    return FVec3D(m(0,0),m(0,1),m(0,2));
}

INLINE_MEMBER FVec3D FLin3D::getVecY() const
{
    return FVec3D(m(1,0),m(1,1),m(1,2));
}

INLINE_MEMBER FVec3D FLin3D::getVecZ() const
{
    return FVec3D(m(2,0),m(2,1),m(2,2));
}

INLINE_MEMBER FMat3 FLin3D::getFMat3() const
{
    return m;
}

INLINE_MEMBER FLin3D & FLin3D::operator*=(const FScale3D & scale)
{
    m.scale(scale.c);
    return *this;
}

INLINE_MEMBER FLin3D & FLin3D::operator*=(const FRotX3D & rot)
{
    m.rot0(rot.cosAlpha,rot.sinAlpha);
    return *this;
}

INLINE_MEMBER FLin3D & FLin3D::operator*=(const FRotY3D & rot)
{
    m.rot1(rot.cosAlpha,rot.sinAlpha);
    return *this;
}

INLINE_MEMBER FLin3D & FLin3D::operator*=(const FRotZ3D & rot)
{
    m.rot2(rot.cosAlpha,rot.sinAlpha);
    return *this;
}

INLINE_MEMBER FLin3D & FLin3D::operator*=(const FShearXZ3D & shear)
{
    m.shear01(shear.c.x0(),shear.c.x1());
    return *this;
}

INLINE_MEMBER FLin3D & FLin3D::operator*=(const FShearZX3D & shear)
{
    m.shear20(shear.c.x0(),shear.c.x1());
    return *this;
}

INLINE_MEMBER FLin3D & FLin3D::operator*=(const FShearYZ3D & shear)
{
    m.shear12(shear.c.x0(),shear.c.x1());
    return *this;
}

INLINE_MEMBER FLin3D & FLin3D::operator*=(const FLin3D & lin)
{
    m *= lin.m;
    return *this;
}

INLINE_GLOBAL FVec3D getVecX(const FLin3D & lin) { return lin.getVecX(); }
INLINE_GLOBAL FVec3D getVecY(const FLin3D & lin) { return lin.getVecY(); }
INLINE_GLOBAL FVec3D getVecZ(const FLin3D & lin) { return lin.getVecZ(); }
INLINE_GLOBAL FMat3 getFMat3(const FLin3D & lin) { return lin.m; }

INLINE_GLOBAL float determinant(const FLin3D & lin)
{
    return determinant(lin);
}

INLINE_GLOBAL FLin3D inverseCopy(const FLin3D & lin, float det)
{
    return FLin3D(inverseCopy(lin.m,det));
}

INLINE_MEMBER void FLin3D::invert(
        )
{
    *this = inverseCopy(*this,determinant(*this));
}

INLINE_GLOBAL ostream & operator<<(ostream & s, const FLin3D & lin)
{
    s << "FLin3D(" << lin.m << ')';
    return s;
}

INLINE_GLOBAL FLin3D inverseCopy(const FLin3D & lin)
{
    return inverseCopy(lin,determinant(lin));
}

INLINE_GLOBAL FLin3D operator*(const FLin3D & lin, const FScale3D & scale)
{
    return FLin3D(prMatScale(lin.m,scale.c));
}

INLINE_GLOBAL FLin3D operator*(const FLin3D & lin, const FRotX3D & rot)
{
    return FLin3D(prMatRot0(lin.m,rot.cosAlpha,rot.sinAlpha));
}

INLINE_GLOBAL FLin3D operator*(const FLin3D & lin, const FRotY3D & rot)
{
    return FLin3D(prMatRot1(lin.m,rot.cosAlpha,rot.sinAlpha));
}

INLINE_GLOBAL FLin3D operator*(const FLin3D & lin, const FRotZ3D & rot)
{
    return FLin3D(prMatRot2(lin.m,rot.cosAlpha,rot.sinAlpha));
}

INLINE_GLOBAL FLin3D operator*(const FLin3D & lin, const FShearXZ3D & shear)
{
    return FLin3D(prMatShear01(lin.m,shear.c.x0(),shear.c.x1()));
}

INLINE_GLOBAL FLin3D operator*(const FLin3D & lin, const FShearZX3D & shear)
{
    return FLin3D(prMatShear20(lin.m,shear.c.x0(),shear.c.x1()));
}

INLINE_GLOBAL FLin3D operator*(const FLin3D & lin, const FShearYZ3D & shear)
{
    return FLin3D(prMatShear12(lin.m,shear.c.x0(),shear.c.x1()));
}

INLINE_GLOBAL FLin3D operator*(const FLin3D & lin1, const FLin3D & lin2)
{
    return FLin3D(lin1.m * lin2.m);
}

INLINE_GLOBAL FLin3D operator*(const FScale3D & scale, const FLin3D & lin)
{
    return FLin3D(prScaleMat(scale.c,lin.m));
}

INLINE_GLOBAL FLin3D operator*(const FRotX3D & rot, const FLin3D & lin)
{
    return FLin3D(prRot0Mat(rot.cosAlpha,rot.sinAlpha,lin.m));
}

INLINE_GLOBAL FLin3D operator*(const FRotY3D & rot, const FLin3D & lin)
{
    return FLin3D(prRot1Mat(rot.cosAlpha,rot.sinAlpha,lin.m));
}

INLINE_GLOBAL FLin3D operator*(const FRotZ3D & rot, const FLin3D & lin)
{
    return FLin3D(prRot2Mat(rot.cosAlpha,rot.sinAlpha,lin.m));
}

INLINE_GLOBAL FLin3D operator*(const FShearXZ3D & shear, const FLin3D & lin)
{
    return FLin3D(prShear01Mat(shear.c.x0(),shear.c.x1(),lin.m));
}

INLINE_GLOBAL FLin3D operator*(const FShearZX3D & shear, const FLin3D & lin)
{
    return FLin3D(prShear20Mat(shear.c.x0(),shear.c.x1(),lin.m));
}

INLINE_GLOBAL FLin3D operator*(const FShearYZ3D & shear, const FLin3D & lin)
{
    return FLin3D(prShear12Mat(shear.c.x0(),shear.c.x1(),lin.m));
}

// ============================================================= class FTrans3D

INLINE_MEMBER FTrans3D::FTrans3D(
	)
{
    TR_FUN0(FVec3D_constructors,FTrans3D);
}

INLINE_MEMBER FTrans3D::FTrans3D(const FCrd3 & crd)
:
    c(crd)
{
    TR_FUN1(FVec3D_constructors,FTrans3D,crd);
}

INLINE_MEMBER FTrans3D::FTrans3D(float xNew, float yNew, float zNew)
:
    c(xNew, yNew, zNew)
{
    TR_FUN3(FVec3D_constructors,FTrans3D,xNew,yNew,zNew);
}

INLINE_MEMBER FTrans3D::FTrans3D(
	const FVec3D & vec
	)
:
    c(vec.c)
{
    TR_FUN1(FVec3D_constructors,FTrans3D,vec);
}

INLINE_MEMBER FTrans3D::FTrans3D(
	const FPnt3D & pntO
	)
:
    c(pntO.c)
{
    TR_FUN1(FVec3D_constructors,FTrans3D, pntO);
}

INLINE_MEMBER FTrans3D::FTrans3D(const FTrans3D & trans)
:
    c(trans.c)
{
    TR_FUN1(FVec3D_constructors,FTrans3D,"FTrans3D");
}

INLINE_MEMBER FTrans3D::~FTrans3D()
{
    TR_MESSAGE(FVec3D_constructors, "~" << *this);
}

INLINE_MEMBER FTrans3D & FTrans3D::operator*=(const FTrans3D & trans)
{
    c += trans.c;
    return *this;
}

INLINE_MEMBER void FTrans3D::invert(
        )
{
    c = -c;
}

INLINE_GLOBAL ostream & operator<<(ostream & s, const FTrans3D & trans)
{
    s << "FTrans3D(" << trans.c.x0() << "," << trans.c.x1() << ","
      << trans.c.x2() << ")";
    return s;
}

INLINE_GLOBAL FTrans3D operator*(const FTrans3D & trans1, const FTrans3D & trans2)
{
    return FTrans3D(trans1.c + trans2.c);
}

INLINE_GLOBAL FTrans3D inverseCopy(const FTrans3D & trans)
{
    return FTrans3D(-trans.c);
}

// ============================================================= class FTrafo3D

INLINE_MEMBER FTrafo3D::FTrafo3D()
{
    TR_FUN0(FVec3D_constructors,FTrafo3D);
}

INLINE_MEMBER FTrafo3D::FTrafo3D(const FTrafo3D & trafo)
:
    m(trafo.m), c(trafo.c)
{
    TR_FUN1(FVec3D_constructors,FTrafo3D,trafo);
}

INLINE_MEMBER FTrafo3D::FTrafo3D(const FMat3 & mat)
:
    m(mat), c(0.0,0.0,0.0)
{
    TR_FUN1(FVec3D_constructors,FTrafo3D,mat);
}

INLINE_MEMBER FTrafo3D::FTrafo3D(const FMat3 & mat, const FCrd3 & crd)
:
    m(mat), c(crd)
{
    TR_FUN2(FVec3D_constructors,FTrafo3D,mat,crd);
}

INLINE_MEMBER FTrafo3D::FTrafo3D(
        float xx, float xy, float xz,
        float yx, float yy, float yz,
        float zx, float zy, float zz,
        float dx, float dy, float dz
        )
:
    m(xx,xy,xz, yx,yy,yz, zx,zy,zz), c(dx,dy,dz)
{
    TR_FUN4(FVec3D_constructors,FTrafo3D,xx,xy,xz,"...");
}

INLINE_MEMBER FTrafo3D::FTrafo3D(
	const FVec3D & vx, const FVec3D & vy, const FVec3D & vz,
	const FPnt3D & pnt
	)
:
    m(vx.c.x0(),vx.c.x1(),vx.c.x2(),
      vy.c.x0(),vy.c.x1(),vy.c.x2(),
      vz.c.x0(),vz.c.x1(),vz.c.x2()),
    c(pnt.c)
{
    TR_FUN4(FVec3D_constructors,FTrafo3D,vx,vy,vz,pnt);
}

INLINE_MEMBER FTrafo3D::FTrafo3D(const FLin3D & lin, const FTrans3D & trans)
:
    m(lin.m), c(trans.c)
{
    TR_FUN2(FVec3D_constructors,FTrafo3D,lin,trans);
}


INLINE_MEMBER FTrafo3D::FTrafo3D(const FScale3D & scale)
:
    m(  scale.c.x0(),   0.0,            0.0,
        0.0,            scale.c.x1(),   0.0,
        0.0,            0.0,            scale.c.x2()    ),
    c(  0.0,            0.0,            0.0             )
{
    TR_FUN1(FVec3D_constructors,FTrafo3D,scale);
}

INLINE_MEMBER FTrafo3D::FTrafo3D(const FRotX3D & rot)
:
    m(  1.0,    0.0,            0.0,
        0.0,    rot.cosAlpha,   rot.sinAlpha,
        0.0,    -rot.sinAlpha,  rot.cosAlpha    ),
    c(  0.0,    0.0,            0.0             )
{
    TR_FUN1(FVec3D_constructors,FTrafo3D,rot);
}

INLINE_MEMBER FTrafo3D::FTrafo3D(const FRotY3D & rot)
:
    m(  rot.cosAlpha,   0.0,    -rot.sinAlpha,
        0.0,            1.0,    0.0,
        rot.sinAlpha,   0.0,    rot.cosAlpha    ),
    c(  0.0,            0.0,    0.0             )
{
    TR_FUN1(FVec3D_constructors,FTrafo3D,rot);
}

INLINE_MEMBER FTrafo3D::FTrafo3D(const FRotZ3D & rot)
:
    m(  rot.cosAlpha,   rot.sinAlpha,   0.0,
        -rot.sinAlpha,  rot.cosAlpha,   0.0,
        0.0,            0.0,            1.0     ),
    c(  0.0,            0.0,            0.0     )
{
    TR_FUN1(FVec3D_constructors,FTrafo3D,rot);
}

INLINE_MEMBER FTrafo3D::FTrafo3D(const FShearXZ3D & shear)
:
    m(  1.0,		0.0,		0.0,
        0.0,            1.0,		0.0,
        shear.c.x0(),	shear.c.x1(),	1.0	),
    c(  0.0,            0.0,            0.0	)
{
    TR_FUN1(FVec3D_constructors,FLin3D,shear);
}

INLINE_MEMBER FTrafo3D::FTrafo3D(const FShearZX3D & shear)
:
    m(  1.0,		0.0,		0.0,
        shear.c.x1(),	1.0,   shear.c.x0(),
        0.0,		0.0,		1.0	),
    c(  0.0,            0.0,            0.0	)
{
    TR_FUN1(FVec3D_constructors,FLin3D,shear);
}

INLINE_MEMBER FTrafo3D::FTrafo3D(const FShearYZ3D & shear)
:
    m(  1.0,   shear.c.x0(),   shear.c.x1(),
        0.0,            1.0,		0.0,
        0.0,		0.0,		1.0	),
    c(  0.0,            0.0,            0.0	)
{
    TR_FUN1(FVec3D_constructors,FLin3D,shear);
}

INLINE_MEMBER FTrafo3D::FTrafo3D(const FRot3D & rot)
:
    c(0.0,0.0,0.0)
{
    float aa = square(rot.c.x0());
    float ab2 = 2 * rot.c.x0() * rot.c.x1();
    float ac2 = 2 * rot.c.x0() * rot.c.x2();
    float ad2 = 2 * rot.c.x0() * rot.c.x3();
    float bb = square(rot.c.x1());
    float bc2 = 2 * rot.c.x1() * rot.c.x2();
    float bd2 = 2 * rot.c.x1() * rot.c.x3();
    float cc = square(rot.c.x2());
    float cd2 = 2 * rot.c.x2() * rot.c.x3();
    float dd = square(rot.c.x3());
    m = FMat3(aa - bb + dd - cc, ab2 - cd2, ac2 + bd2,
             ab2 + cd2, bb - aa + dd - cc, bc2 - ad2,
             ac2 - bd2, bc2 + ad2, cc - aa + dd - bb); 
    TR_FUN1(FVec3D_constructors,FTrafo3D,rot);
}

INLINE_MEMBER FTrafo3D::FTrafo3D(const FLin3D & lin)
:
    m(lin.m), c(0.0,0.0,0.0)
{
    TR_FUN1(FVec3D_constructors,FTrafo3D,lin);
}

INLINE_MEMBER FTrafo3D::FTrafo3D(const FTrans3D & trans)
:
    m(1.0,0.0,0.0, 0.0,1.0,0.0, 0.0,0.0,1.0), c(trans.c)
{
    TR_FUN1(FVec3D_constructors,FTrafo3D,trans);
}

INLINE_MEMBER FTrafo3D::~FTrafo3D()
{
    TR_MESSAGE(FVec3D_constructors,"~" << *this);
}

INLINE_MEMBER FVec3D FTrafo3D::getVecX() const
{
    return FVec3D(m(0,0),m(0,1),m(0,2));
}

INLINE_MEMBER FVec3D FTrafo3D::getVecY() const
{
    return FVec3D(m(1,0),m(1,1),m(1,2));
}

INLINE_MEMBER FVec3D FTrafo3D::getVecZ() const
{
    return FVec3D(m(2,0),m(2,1),m(2,2));
}

INLINE_MEMBER FPnt3D FTrafo3D::getPntO() const
{
    return FPnt3D(c);
}

INLINE_MEMBER FLin3D FTrafo3D::getLin() const
{
    return FLin3D(m);
}

INLINE_MEMBER FTrans3D FTrafo3D::getTrans() const
{
    return FTrans3D(c);
}

INLINE_MEMBER FMat3 FTrafo3D::getFMat3() const
{
    return m;
}

INLINE_MEMBER FMat4 FTrafo3D::getFMat4() const
{
    return FMat4(m(0,0), m(0,1), m(0,2), 0.0,
		m(1,0), m(1,1), m(1,2), 0.0,
		m(2,0), m(2,1), m(2,2), 0.0,
		c.x0(), c.x1(), c.x2(), 1.0);
}

INLINE_MEMBER FTrafo3D & FTrafo3D::operator*=(const FScale3D & scale)
{
    m.scale(scale.c);
    c.scale(scale.c);
    return *this;
}

INLINE_MEMBER FTrafo3D & FTrafo3D::operator*=(const FRotX3D & rot)
{
    m.rot0(rot.cosAlpha,rot.sinAlpha);
    c.rot0(rot.cosAlpha,rot.sinAlpha);
    return *this;
}

INLINE_MEMBER FTrafo3D & FTrafo3D::operator*=(const FRotY3D & rot)
{
    m.rot1(rot.cosAlpha,rot.sinAlpha);
    c.rot1(rot.cosAlpha,rot.sinAlpha);
    return *this;
}

INLINE_MEMBER FTrafo3D & FTrafo3D::operator*=(const FRotZ3D & rot)
{
    m.rot2(rot.cosAlpha,rot.sinAlpha);
    c.rot2(rot.cosAlpha,rot.sinAlpha);
    return *this;
}

INLINE_MEMBER FTrafo3D & FTrafo3D::operator*=(const FShearXZ3D & shear)
{
    m.shear01(shear.c.x0(),shear.c.x1());
    c.shear01(shear.c.x0(),shear.c.x1());
    return *this;
}

INLINE_MEMBER FTrafo3D & FTrafo3D::operator*=(const FShearZX3D & shear)
{
    m.shear20(shear.c.x0(),shear.c.x1());
    c.shear20(shear.c.x0(),shear.c.x1());
    return *this;
}

INLINE_MEMBER FTrafo3D & FTrafo3D::operator*=(const FShearYZ3D & shear)
{
    m.shear12(shear.c.x0(),shear.c.x1());
    c.shear12(shear.c.x0(),shear.c.x1());
    return *this;
}

INLINE_MEMBER FTrafo3D & FTrafo3D::operator*=(const FTrans3D & trans)
{
    c += trans.c;
    return *this;
}

INLINE_MEMBER FTrafo3D & FTrafo3D::operator*=(const FLin3D & lin)
{
    m *= lin.m;
    c *= lin.m;
    return *this;
}

INLINE_MEMBER FTrafo3D & FTrafo3D::operator*=(const FTrafo3D & trafo)
{
    m *= trafo.m;
    c *= trafo.m;
    c += trafo.c;
    return *this;
}

INLINE_GLOBAL FVec3D getVecX(const FTrafo3D & trafo) { return trafo.getVecX(); }
INLINE_GLOBAL FVec3D getVecY(const FTrafo3D & trafo) { return trafo.getVecY(); }
INLINE_GLOBAL FVec3D getVecZ(const FTrafo3D & trafo) { return trafo.getVecZ(); }
INLINE_GLOBAL FPnt3D getPntO(const FTrafo3D & trafo) { return trafo.getPntO(); }
INLINE_GLOBAL FLin3D getLin(const FTrafo3D & trafo) { return trafo.getLin(); }
INLINE_GLOBAL FTrans3D getTrans(const FTrafo3D & trafo) { return trafo.getTrans(); }
INLINE_GLOBAL FMat3 getFMat3(const FTrafo3D & trafo) { return trafo.m; }
INLINE_GLOBAL FMat4 getFMat4(const FTrafo3D & trafo) { return trafo.getFMat4(); }

INLINE_GLOBAL float determinant(const FTrafo3D & trafo)
{
    return determinant(trafo);
}

INLINE_GLOBAL FTrafo3D inverseCopy(const FTrafo3D & trafo, float det)
{
    FTrafo3D inv;

    inv.m = inverseCopy(trafo.m,det);

    inv.c = - (trafo.c * inv.m);
                 
    return inv;
}

INLINE_MEMBER void FTrafo3D::invert(
        )
{
    float det = determinant(*this);
    *this = inverseCopy(*this,det);
}

INLINE_GLOBAL ostream & operator<<(ostream & s, const FTrafo3D & trafo)
{
    s << "FTrafo3D(" << trafo.m << ',' << trafo.c << ')';
    return s;
}

INLINE_GLOBAL FTrafo3D inverseCopy(const FTrafo3D & trafo)
{
    float det = determinant(trafo);
    return inverseCopy(trafo,det);
}

INLINE_GLOBAL FTrafo3D operator*(const FTrafo3D & trafo, const FScale3D & scale)
{
    return FTrafo3D(prMatScale(trafo.m,scale.c),
                   prCrdScale(trafo.c,scale.c));
}

INLINE_GLOBAL FTrafo3D operator*(const FTrafo3D & trafo, const FRotX3D & rot)
{
    return FTrafo3D(prMatRot0(trafo.m,rot.cosAlpha,rot.sinAlpha),
                   prCrdRot0(trafo.c,rot.cosAlpha,rot.sinAlpha));
}

INLINE_GLOBAL FTrafo3D operator*(const FTrafo3D & trafo, const FRotY3D & rot)
{
    return FTrafo3D(prMatRot1(trafo.m,rot.cosAlpha,rot.sinAlpha),
                   prCrdRot1(trafo.c,rot.cosAlpha,rot.sinAlpha));
}

INLINE_GLOBAL FTrafo3D operator*(const FTrafo3D & trafo, const FRotZ3D & rot)
{
    return FTrafo3D(prMatRot2(trafo.m,rot.cosAlpha,rot.sinAlpha),
                   prCrdRot2(trafo.c,rot.cosAlpha,rot.sinAlpha));
}

INLINE_GLOBAL FTrafo3D operator*(const FTrafo3D & trafo, const FShearXZ3D & shear)
{
    return FTrafo3D(prMatShear01(trafo.m,shear.c.x0(),shear.c.x1()),
		   prCrdShear01(trafo.c,shear.c.x0(),shear.c.x1()));
}

INLINE_GLOBAL FTrafo3D operator*(const FTrafo3D & trafo, const FShearZX3D & shear)
{
    return FTrafo3D(prMatShear20(trafo.m,shear.c.x0(),shear.c.x1()),
    		   prCrdShear20(trafo.c,shear.c.x0(),shear.c.x1()));
}

INLINE_GLOBAL FTrafo3D operator*(const FTrafo3D & trafo, const FShearYZ3D & shear)
{
    return FTrafo3D(prMatShear12(trafo.m,shear.c.x0(),shear.c.x1()),
		   prCrdShear12(trafo.c,shear.c.x0(),shear.c.x1()));
}

INLINE_GLOBAL FTrafo3D operator*(const FTrafo3D & trafo, const FTrans3D & trans)
{
    return FTrafo3D(trafo.m, trafo.c + trans.c);
}

INLINE_GLOBAL FTrafo3D operator*(const FTrafo3D & trafo, const FLin3D & lin)
{
    return FTrafo3D(trafo.m * lin.m, trafo.c * lin.m);
}

INLINE_GLOBAL FTrafo3D operator*(const FScale3D & scale, const FTrafo3D & trafo)
{
    return FTrafo3D(prScaleMat(scale.c,trafo.m),trafo.c);
}

INLINE_GLOBAL FTrafo3D operator*(const FRotX3D & rot, const FTrafo3D & trafo)
{
    return FTrafo3D(prRot0Mat(rot.cosAlpha,rot.sinAlpha,trafo.m),trafo.c);
}

INLINE_GLOBAL FTrafo3D operator*(const FRotY3D & rot, const FTrafo3D & trafo)
{
    return FTrafo3D(prRot1Mat(rot.cosAlpha,rot.sinAlpha,trafo.m),trafo.c);
}

INLINE_GLOBAL FTrafo3D operator*(const FRotZ3D & rot, const FTrafo3D & trafo)
{
    return FTrafo3D(prRot2Mat(rot.cosAlpha,rot.sinAlpha,trafo.m),trafo.c);
}

INLINE_GLOBAL FTrafo3D operator*(const FShearXZ3D & shear, const FTrafo3D & trafo)
{
    return FTrafo3D(prShear01Mat(shear.c.x0(),shear.c.x1(),trafo.m),
		   trafo.c);
}

INLINE_GLOBAL FTrafo3D operator*(const FShearZX3D & shear, const FTrafo3D & trafo)
{
    return FTrafo3D(prShear20Mat(shear.c.x0(),shear.c.x1(),trafo.m),
		   trafo.c);
}

INLINE_GLOBAL FTrafo3D operator*(const FShearYZ3D & shear, const FTrafo3D & trafo)
{
    return FTrafo3D(prShear12Mat(shear.c.x0(),shear.c.x1(), trafo.m),
		   trafo.c);
}

INLINE_GLOBAL FTrafo3D operator*(const FTrans3D & trans, const FTrafo3D & trafo)
{
    return FTrafo3D(trafo.m,trans.c * trafo.m + trafo.c);
}

INLINE_GLOBAL FTrafo3D operator*(const FLin3D & lin, const FTrafo3D & trafo)
{
    return FTrafo3D(lin.m * trafo.m,trafo.c);
}

INLINE_GLOBAL FTrafo3D operator*(const FTrafo3D & trafo1, const FTrafo3D & trafo2)
{
    return FTrafo3D(trafo1.m * trafo2.m, trafo1.c * trafo2.m + trafo2.c);
}

} // extern "C++"

// ===========================================================================

