// ===========================================================================
//  Copyright 1994 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       Vec
//  TYPE:       c++ header
//  PROJECT:    VEGA Graphics Library
//  CONTENT:    vector tools
//  VERSION:    1.0.0
// ===========================================================================
//  AUTHORS:    rft     Robert F. Tobler
// ===========================================================================
//  HISTORY:
//
//  30-sep-94 14:08:16  rft     last modification
//  13-apr-94 09:40:51  rft     created
// ===========================================================================
//  MANUAL:
//
//  DESCRIPTION
//      This module provides conversion functions between the various types
//      of vectors.
//
//  SEE ALSO
//      VEGA, GraphicsLibrary
// ===========================================================================

#ifndef _GRAPHICS_VEC_H_
#define _GRAPHICS_VEC_H_

#include "VEGA/Graphics/globals.h"
#include "VEGA/Graphics/vec2d.h"
#include "VEGA/Graphics/vec3d.h"
#include "VEGA/Graphics/fvec2d.h"
#include "VEGA/Graphics/fvec3d.h"

extern "C++"
{

Vec2D getVec2D(
	const FVec2D & vec
	);
Pnt2D getPnt2D(
	const FPnt2D & pnt
	);
Trafo2D getTrafo2D(
	const FTrafo2D & trafo
	);

FVec2D getFVec2D(
	const Vec2D & vec
	);
FPnt2D getFPnt2D(
	const Pnt2D & pnt
	);
FTrafo2D getFTrafo2D(
	const Trafo2D & trafo
	);

Vec3D getVec3D(
	const FVec3D & vec
	);
Pnt3D getPnt3D(
	const FPnt3D & pnt
	);
Trafo3D getTrafo3D(
	const FTrafo3D & trafo
	);

FVec3D getFVec3D(
	const Vec3D & vec
	);
FPnt3D getFPnt3D(
	const Pnt3D & pnt
	);
FTrafo3D getFTrafo3D(
	const Trafo3D & trafo
	);

Pnt2D getParallelProjectedX(
	const Pnt3D & pnt	
	);
Pnt2D getParallelProjectedY(
	const Pnt3D & pnt
	);
Pnt2D getParallelProjectedZ(
	const Pnt3D & pnt
	);
Pnt2D getCentralProjectedZ(
	const Pnt3D & pnt
	);

FPnt2D getParallelProjectedX(
	const FPnt3D & pnt
	);
FPnt2D getParallelProjectedY(
	const FPnt3D & pnt
	);
FPnt2D getParallelProjectedZ(
	const FPnt3D & pnt
	);
FPnt2D getCentralProjectedZ(
	const FPnt3D & pnt
	);

} // extern "C++"

#ifndef VEC_NO_INLINE

#define INLINE_GLOBAL COMPILER_INLINE_GLOBAL
#define INLINE_MEMBER COMPILER_INLINE_MEMBER

#include "VEGA/Graphics/vec.ipp"

#undef INLINE_GLOBAL
#undef INLINE_MEMBER

#endif

#endif // _GRAPHICS_VEC_H_
// ===========================================================================
