// ===========================================================================
//  Copyright 1994 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       Vec
//  TYPE:       c++ inline
//  PROJECT:    VEGA Graphics Library
//  CONTENT:    vector tools
//  VERSION:    1.0.0
// ===========================================================================
//  AUTHORS:    rft     Robert F. Tobler
// ===========================================================================
//  HISTORY:
//
//  30-sep-94 14:08:16  rft     last modification
//  13-apr-94 09:40:51  rft     created
// ===========================================================================

extern "C++"
{

INLINE_GLOBAL Vec2D getVec2D(const FVec2D & vec)
{
    return Vec2D(vec.x(),vec.y());
}

INLINE_GLOBAL Pnt2D getPnt2D(const FPnt2D & pnt)
{
    return Pnt2D(pnt.x(),pnt.y());
}

INLINE_GLOBAL Trafo2D getTrafo2D(const FTrafo2D & trafo)
{
    return
	Trafo2D
	(
	    getVec2D(trafo.getVecX()),
	    getVec2D(trafo.getVecY()),
	    getPnt2D(trafo.getPntO())
	);
}

INLINE_GLOBAL FVec2D getFVec2D(const Vec2D & vec)
{
    return FVec2D(vec.x(),vec.y());
}

INLINE_GLOBAL FPnt2D getFPnt2D(const Pnt2D & pnt)
{
    return FPnt2D(pnt.x(), pnt.y());
}

INLINE_GLOBAL FTrafo2D getFTrafo2D(const Trafo2D & trafo)
{
    return
	FTrafo2D
	(
	    getFVec2D(trafo.getVecX()),
	    getFVec2D(trafo.getVecY()),
	    getFPnt2D(trafo.getPntO())
	);
}

INLINE_GLOBAL Vec3D getVec3D(const FVec3D & vec)
{
    return Vec3D(vec.x(),vec.y(),vec.z());
}

INLINE_GLOBAL Pnt3D getPnt3D(const FPnt3D & pnt)
{
    return Pnt3D(pnt.x(), pnt.y(), pnt.z());
}

INLINE_GLOBAL Trafo3D getTrafo3D(const FTrafo3D & trafo)
{
    return
	Trafo3D
	(
	    getVec3D(trafo.getVecX()),
	    getVec3D(trafo.getVecY()),
	    getVec3D(trafo.getVecZ()),
	    getPnt3D(trafo.getPntO())
	);
}


INLINE_GLOBAL FVec3D getFVec3D(const Vec3D & vec)
{
    return FVec3D(vec.x(),vec.y(),vec.z());
}

INLINE_GLOBAL FPnt3D getFPnt3D(const Pnt3D & pnt)
{
    return FPnt3D(pnt.x(), pnt.y(), pnt.z());
}

INLINE_GLOBAL FTrafo3D getFTrafo3D(const Trafo3D & trafo)
{
    return
	FTrafo3D
	(
	    getFVec3D(trafo.getVecX()),
	    getFVec3D(trafo.getVecY()),
	    getFVec3D(trafo.getVecZ()),
	    getFPnt3D(trafo.getPntO())
	);
}

INLINE_GLOBAL Pnt2D getParallelProjectedX(const Pnt3D & pnt)
{
    return Pnt2D(pnt.y(),pnt.z());
}

INLINE_GLOBAL Pnt2D getParallelProjectedY(const Pnt3D & pnt)
{
    return Pnt2D(pnt.z(),pnt.x());
}

INLINE_GLOBAL Pnt2D getParallelProjectedZ(const Pnt3D & pnt)
{
    return Pnt2D(pnt.x(),pnt.y());
}

INLINE_GLOBAL Pnt2D getCentralProjectedZ(const Pnt3D & pnt)
{
    return Pnt2D(pnt.x()/pnt.z(),pnt.y()/pnt.z());
}

INLINE_GLOBAL FPnt2D getParallelProjectedX(const FPnt3D & pnt)
{
    return FPnt2D(pnt.y(),pnt.z());
}

INLINE_GLOBAL FPnt2D getParallelProjectedY(const FPnt3D & pnt)
{
    return FPnt2D(pnt.z(),pnt.x());
}

INLINE_GLOBAL FPnt2D getParallelProjectedZ(const FPnt3D & pnt)
{
    return FPnt2D(pnt.x(),pnt.y());
}

INLINE_GLOBAL FPnt2D getCentralProjectedZ(const FPnt3D & pnt)
{
    return FPnt2D(pnt.x()/pnt.z(),pnt.y()/pnt.z());
}

} // extern "C++"

// ===========================================================================
