// ===========================================================================
//  Copyright 1994 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       Bool
//  TYPE:       c++ inline
//  PROJECT:    VEGA Base Library
//  CONTENT:    Boolean type
//  VERSION:    1.0.0
// ===========================================================================
//  AUTHORS:    rft     Robert F. Tobler
// ===========================================================================
//  HISTORY:
//
//  30-sep-94 13:40:13  rft     last modification
//  21-mar-94 23:40:17  rft     created
// ===========================================================================

extern "C++"
{

INLINE_MEMBER Bool::Bool(State newState)
:
    state((int)newState)
{}

INLINE_MEMBER int Bool::isTrue() const
{
    return (state);
}

INLINE_MEMBER int Bool::isFalse() const
{
    return (state ^ (int)Bool::yes);
}

INLINE_MEMBER int Bool::isNotTrue() const
{
    return (state ^ (int)Bool::yes);
}

INLINE_MEMBER int Bool::isNotFalse() const
{
    return (state);
}

INLINE_GLOBAL int isTrue(Bool b)
{
    return (b.state);
}

INLINE_GLOBAL int isFalse(Bool b)
{
    return (b.state ^ (int)Bool::yes);
}

INLINE_GLOBAL int isNotTrue(Bool b)
{
    return (b.state ^ (int)Bool::yes);
}

INLINE_GLOBAL int isNotFalse(Bool b)
{
    return (b.state);
}

INLINE_GLOBAL Bool operator!(Bool b)
{
    return Bool((Bool::State)(b.state ^ (int)Bool::yes));
}

INLINE_GLOBAL Bool operator||(Bool b1, Bool b2)
{
    return Bool((Bool::State)(b1.state | b2.state));
}

INLINE_GLOBAL Bool operator&&(Bool b1, Bool b2)
{
    return Bool((Bool::State)(b1.state & b2.state));
}

INLINE_GLOBAL Bool operator==(Bool b1, Bool b2)
{
    return Bool((Bool::State)(b1.state ^ b2.state ^ (int)Bool::yes));
}

INLINE_GLOBAL Bool operator!=(Bool b1, Bool b2)
{
    return Bool((Bool::State)(b1.state ^ b2.state));
}

INLINE_GLOBAL ostream & operator<<(ostream & s, Bool b)
{
    s << (b.state == (int)Bool::no ? "Bool::no" : "Bool::yes");
    return s;
}

} // extern "C++"

// ===========================================================================
