// ===========================================================================
//  Copyright 1994 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       Error
//  TYPE:       c++ inline
//  PROJECT:    VEGA Base Library
//  CONTENT:    simple error handler
//  VERSION:    1.0.0
// ===========================================================================
//  AUTHORS:    hl      Helwig L"offelmann
//              rft     Robert F. Tobler
// ===========================================================================
//  HISTORY:
//
//  30-sep-94 13:40:13  rft     last modification
//  04-jan-94 22:25:00  hl      created
// ===========================================================================

extern "C++"
{

INLINE_MEMBER ErrorHandler::ErrorHandler(
        const char * newName
        )
:
    name(newName),
    state(errorAndExit),
    warnings(enabled),
    errorNumber(EH_NO_ERROR),
    exitFunction(exit),
    exitCode(-1),
    localStream(0)
{}

INLINE_MEMBER void ErrorHandler::doExit() const
{
    if (exitFunction == _exit)
        ::_exit(exitCode);
    else
        ::exit(exitCode);
}

INLINE_MEMBER void ErrorHandler::condExit() const
{
    if (state == errorAndExit)
        doExit();
}

INLINE_MEMBER int ErrorHandler::printErrors() const
{
    return (state != useErrorNumber);
}

INLINE_MEMBER int ErrorHandler::printWarnings() const
{
    return (warnings == enabled);
}

INLINE_GLOBAL ostream & operator<<(ostream & s, const ErrorHandler & handler)
{
    s << handler.name;
    return s;
}

INLINE_MEMBER ErrorHandler::State ErrorHandler::setState(State newState)
{
    State oldState = state;
    state = newState;
    return oldState;
}

INLINE_MEMBER ErrorHandler::Warnings ErrorHandler::setWarnings(
	Warnings newWarnings
	)
{
    Warnings oldWarnings = warnings;
    warnings = newWarnings;
    return oldWarnings;
}

INLINE_MEMBER ErrorHandler::ExitFunction ErrorHandler::setExitFunction(
	ExitFunction newExitFunction
	)
{
    ExitFunction oldExitFunction = exitFunction;
    exitFunction = newExitFunction;
    return oldExitFunction;
}

INLINE_MEMBER int ErrorHandler::setExitCode(int newExitCode)
{
    int oldExitCode = exitCode;
    exitCode = newExitCode;
    return oldExitCode;
}

INLINE_MEMBER void ErrorHandler::setErrorNumber(int newErrorNumber)
{
    errorNumber = newErrorNumber;
    lastErrorHandlerPtr = this;
}

INLINE_MEMBER int ErrorHandler::getErrorNumber() const
{
    return errorNumber;
}

INLINE_MEMBER ostream & ErrorHandler::getStream() const
{
    if (localStream != 0)
        return *localStream;
    else
        return *commonStream;
}

INLINE_MEMBER void ErrorHandler::enableLocalStream(ostream & s)
{
    localStream = &s;
}

INLINE_MEMBER void ErrorHandler::disableLocalStream()
{
    localStream = 0;
}

INLINE_MEMBER void ErrorHandler::setCommonStream(ostream & s)
{
    commonStream = &s;
}

INLINE_MEMBER void ErrorHandler::setLastErrorHandlerPtr(
    ErrorHandler * errorHandlerPtr
    )
{
    lastErrorHandlerPtr = errorHandlerPtr;
}

INLINE_MEMBER ErrorHandler * ErrorHandler::getLastErrorHandlerPtr()
{
    return lastErrorHandlerPtr;
}

} // extern "C++"

// ===========================================================================

