// ===========================================================================
//  Copyright 1994 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       System
//  TYPE:       c++ header
//  PROJECT:    VEGA Base Library
//  CONTENT:    system data types
//  VERSION:    1.0.0
// ===========================================================================
//  AUTHORS:    rft     Robert F. Tobler
// ===========================================================================
//  HISTORY:
//
//  30-sep-94 13:40:13  rft     last modification
//  10-Feb-93           rft     created
// ===========================================================================
//  MANUAL:
//
//  DESCRIPTION
//      This module defines a few basic types and constants.
//
//  SEE ALSO
//      VEGA, BaseLibrary
// ===========================================================================
#ifndef _BASE_SYSTEM_HH_
#define _BASE_SYSTEM_HH_

#include <limits.h>

#include "VEGA/Base/Defines.h"

// ---------------------------------------------------------------------------
//  Define (U)Int[8|16|32|64] in the system specific file.
// ---------------------------------------------------------------------------

#ifdef NEXT
#  include "VEGA/Base/SysNEXT.h"
#elif SGI
#  include "VEGA/Base/SysSGI.h"
#elif SGI_GCC
#  include "VEGA/Base/SysSGI_GCC.h"
#elif LINUX
#  include "VEGA/Base/SysLINUX.h"
#elif __BCPLUSPLUS__
#  include "VEGA/Base/SysDOS_BCC.h"
#endif

extern "C++"
{

// ---------------------------------------------------------------------------
//
//  This is the data type which is used by routines that directly manipulate
//  memory in order to do pointer arithmetic on memory pointers.
//

typedef unsigned char   Byte;           // general memory

// ------------------------------------------------ the longest supported type

#ifdef LONGLONG

typedef long long               VeryLong;
typedef unsigned long long      UVeryLong;

#else

typedef long                    VeryLong;
typedef unsigned long           UVeryLong;

#endif

// ---------------------------------------------------------------------------
//      constants defined as macros
// ---------------------------------------------------------------------------

#ifdef INT8_NUMBITS
#define INT8_MIN        ((Int8) -128)
#define INT8_MAX        ((Int8) +127)
#define UINT8_MAX       ((UInt8) 0xff)
#endif

#ifdef INT16_NUMBITS
#define INT16_MIN       ((Int16) -32768)
#define INT16_MAX       ((Int16) +32767)
#define UINT16_MAX      ((UInt16) 0xffff)
#endif

#ifdef INT32_NUMBITS
#define INT32_MIN       ((Int32) -2147483648L)
#define INT32_MAX       ((Int32) +2147483647L)
#define UINT32_MAX      ((UInt32) 0xffffffffL)
#endif

#ifdef INT64_NUMBITS
#define INT64_MIN       ((Int64) -9223372036854775808LL)
#define INT64_MAX       ((Int64) +9223372036854775807LL)
#define UINT64_MAX      ((UInt64) 0xffffffffffffffffULL)
#endif

//
//  These are the preferred functions for testing pointers:
//

int isNIL(
	const void * ptr
	);
int isNotNIL(
	const void * ptr
	);

//
//  This is the preferred macro for generating typed NIL-pointers:
//

#define NIL(pointerType) ((pointerType)0)

} // extern "C++"

#ifndef SYSTEM_NO_INLINE

#define INLINE_GLOBAL COMPILER_INLINE_GLOBAL
#define INLINE_MEMBER COMPILER_INLINE_MEMBER

#include "VEGA/Base/System.I"

#undef INLINE_GLOBAL
#undef INLINE_MEMBER

#endif


#endif // _BASE_SYSTEM_HH_
// ===========================================================================
