// ===========================================================================
//  Copyright 1994 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       Test
//  TYPE:       c++ code
//  PROJECT:    VEGA Base Library
//  CONTENT:    simple test support
//  VERSION:    1.0.0
// ===========================================================================
//  AUTHORS:    rft     Robert F. Tobler
// ===========================================================================
//  HISTORY:
//
//  30-sep-94 13:40:13  rft     last modification
//  16-mar-94 20:41:49  rft     created
// ===========================================================================

extern "C"
{
#include <stdlib.h>
#include <string.h>
}

#include <iostream.h>

#include "VEGA/Base/Test.h"

TestSuite::TestSuite(const char * name)
{
    cout << "\nTESTING " << name << ":\n";
    tests = 0;
    bugs = 0;
}

TestSuite::~TestSuite()
{
    if (bugs > 0)
    {
        cout << "\nWARNING: " << bugs << " of " << tests
             << " TESTS FAILED!!!\07\n\n" << flush;
	exit(-1);
    }
    else
        cout << "\nALL TESTS SUCCEEDED!!!\n\n";
}

TestModule::TestModule(TestSuite & suite, const char * name)
:
    testSuite(suite)
{
    cout << "\n  TESTING MODULE " << name << ":\n\n";
    tests = 0;
    bugs = 0;
}

TestModule::~TestModule()
{
    testSuite.tests += tests;
    testSuite.bugs += bugs;
}

static const char * dotString =
    "..................................................";

static const char * Dots(int num)
{
    return (dotString + 50 - num);
}

void TestModule::test(const char * name, int result)
{
    Test localTest(*this,name,Test::failed);
    
    if (result)
        localTest.setPassed();
}

Test::Test(TestModule & module, const char * name, Test::State newState)
:
    testModule(module),
    testName(name),
    state(newState)
{
    testModule.tests++;
}

Test::~Test()
{
    const char * dots = Dots(48-strlen(testName));
    cout << "    " << testName << dots;
    if (state == passed)
    {
        cout << "ok\n";
    }
    else
    {
        cout << "FAILED!\07\n";
        testModule.bugs++;
    }
}

void Test::setFailed()
{
    state = failed;
}

void Test::setPassed()
{
    state = passed;
}

// ===========================================================================

