// ===========================================================================
//  Copyright 1994 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       Test
//  TYPE:       c++ header
//  PROJECT:    VEGA Base Library
//  CONTENT:    simple test support
//  VERSION:    1.0.0
// ===========================================================================
//  AUTHORS:    rft     Robert F. Tobler
// ===========================================================================
//  HISTORY:
//
//  30-sep-94 13:40:13  rft     last modification
//  16-mar-94 20:41:49  rft     created
// ===========================================================================
//  MANUAL:
//
//  DESCRIPTION:
//      This module contains classes for writing test code. The modules take
//      care of the statistics and output of success/failure of the tests.
//
//	To find out how these can be used, the test code of the library can
//	be taken as an example.
//
//  SEE ALSO
//      VEGA, BaseLibrary
// ===========================================================================
#ifndef _BASE_TEST_H_
#define _BASE_TEST_H_

#ifdef __GNUC__
#pragma interface
#endif

#include "VEGA/Base/System.h"

extern "C++"
{

// =========================================================== class TestSuite

class TestSuite
{
    friend class TestModule;
public:

    TestSuite(
	const char * name
	);
    ~TestSuite(
	);

private:
    int tests;
    int bugs;
};

class TestModule
{
    friend class Test;
public:

    TestModule(
	TestSuite & suite,
	const char * name
	);
    ~TestModule(
	);

    void test(
	const char * name,
	int result
	);

private:
    int tests;
    int bugs;
    TestSuite & testSuite;
};

class Test
{
public:

    enum State
    {
	failed = 0,
	passed = 1
    };

    Test(
	TestModule & module,
	const char * name,
	State newState
	);
    ~Test(
	);

    void setFailed(
	);
    void setPassed(
	);

private:
    const char * testName;
    State state;
    TestModule & testModule;
};

} // extern "C++"

#endif // _BASE_TEST_H_
// ===========================================================================

