// ===========================================================================
//  Copyright 1994 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       LibraryTest
//  TYPE:       c++ code
//  PROJECT:    VEGA Graphics Library
//  CONTENT:    VEGA Graphics Library Test
//  VERSION:    1.0.0
// ===========================================================================
//  AUTHORS:    rft     Robert F. Tobler
// ===========================================================================
//  HISTORY:
//
//  30-sep-94 14:08:57  rft     last modification
//  30-sep-94 14:08:57  rft     created
// ===========================================================================

extern "C"
{
#include <string.h>
}


#include <VEGA/Base/Library.h>
#include <VEGA/Math/Globals.h>
#include <VEGA/Graphics/Globals.h>

void testVec3D(TestSuite & suite);

Bool allTests = Bool::yes;

int doTest(const OptionString & option, const char * name)
{
    if (isTrue(allTests) ||
	strcmp(option.getValue(), name) == 0)
	return 1;
    else
	return 0;
}

int main(int argc, char * argv[])
{
    OptionString singleOpt("single",     "s");

    TR_CODE(OptionFlag debugOpt("debug",         "d"));
    TR_CODE(OptionFlag nCrd3c("noCrd3constructors", "nCrd3c"));
    TR_CODE(OptionFlag nCrd4c("noCrd4constructors", "nCrd4c"));
    TR_CODE(OptionFlag nVec3Dc("noVec3Dconstructors", "nVec3Dc"));

    Option::parse(argc,argv);

    TR_CODE(if (debugOpt.isSpecified()) TR_ON(VEGA_Graphics_tracers));
    TR_CODE(if (nCrd3c.isSpecified()) TR_OFF(Crd3_constructors));
    TR_CODE(if (nCrd4c.isSpecified()) TR_OFF(Crd4_constructors));
    TR_CODE(if (nVec3Dc.isSpecified()) TR_OFF(Vec3D_constructors));

    TestSuite suite("VEGA Graphics Library");
    
    if (singleOpt.isSpecified()) allTests = Bool::no;
    
    if (doTest(singleOpt,"Vec3D"))		testVec3D(suite);

    TR_CODE(if (debugOpt.isSpecified()) TR_OFF(VEGA_Graphics_tracers));

    return (0);
}
