// ===========================================================================
//  Copyright 1994 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       testVec3D
//  TYPE:       c++ code
//  PROJECT:    VEGA Graphics Library
//  CONTENT:    test module Vec3D
//  VERSION:    1.0.0
// ===========================================================================
//  AUTHORS:    rft     Robert F. Tobler
// ===========================================================================
//  HISTORY:
//
//  30-sep-94 14:08:57  rft     last modification
//  30-sep-94 14:08:57  rft     created
// ===========================================================================

#include <VEGA/Base/Bool.h>
#include <VEGA/Base/Test.h>
#include <VEGA/Misc/OutTools.h>
#include <VEGA/Math/Math.h>
#include <VEGA/Graphics/Vec3D.h>

void testVec3D(TestSuite & suite)
{
    TestModule module(suite, "Vec3D");

    // -------------------------------------------------------------- Rot3D #1
    {   
        Test test(module, "Rot3D #1", Test::failed);

        RotX3D rotx(45 DEGREES);
        RotY3D roty(45 DEGREES);
        RotZ3D rotz(45 DEGREES);

        Rot3D rot = Rot3D::identity;
        Lin3D lin = Lin3D::identity;
        
        rot *= rotx;
        rot *= roty;
        rot *= rotz;
        
        lin *= rotx;
        lin *= roty;
        lin *= rotz;
        
        Vec3D vec(1,2,3);
        
        Vec3D vec1 = vec * lin;
        Vec3D vec2 = vec * Lin3D(rot);

        double dist = squaredLength(vec1 - vec2);

        if (isTiny(dist))
            test.setPassed();
    }

    // ------------------------------------------------------------ Rot3D back
    {   
        Test backTest(module, "Rot3D back", Test::passed);
        Test getAlphaTest(module, "Rot3D::getAlpha", Test::passed);

        Rot3D rot;
	Rot3D rot2;
	double alpha;
	double rotAlpha;
	double diff;
	Vec3D rotVec;
	Vec3D vec(1,2,3);
	Vec3D vec1;
	
	OutProgress progress(360);
	        
	for (int deg = 0; deg < 360; deg ++)
	{
	    cout << progress.set(deg);

	    for (int x = -1; x < 2; x++)
		for (int y = -1; y < 2; y++)
		    for (int z = -1; z < 2; z++)
			if (x != 0 || y != 0 || z != 0)
			{
			    alpha = deg DEGREES;
			    rotVec = Vec3D(float(x),float(y),float(z));
			    rot = Rot3D(rotVec,alpha);
			    rot2 = rot * Rot3D(rotVec,-alpha);

			    rotAlpha = rot.getAlpha();

			    diff = square(alpha - rotAlpha);
			    

			    if (! isTiny(diff))
			    {
				cout << rotVec << "," << deg << ": rot = "
				     << rot << "\n";
				getAlphaTest.setFailed();
			    }

			    vec1 = vec * Lin3D(rot2);
		    
			    diff = squaredLength(vec1 - vec);
		    
			    if (! isTiny(diff))
			    {
				cout << rotVec << "," << alpha << ":"
					<< vec << " != " << vec1 << ","
					<< diff << "\n";
				backTest.setFailed();
			    }
			}
	}
    }
    {   
        Test rotationTest(module, "Rot3D vector rotation", Test::passed);

	Rot3D rot;
	Vec3D vec(-1,0,0);
	Rot3D vecRot(vec);
	Vec3D rotVec;
	Lin3D lin;
	Rot3D   outRot;
	double alpha;
	Vec3D	vec1, vec2;

	OutProgress progress(360);
	        
	for (int deg = 0; deg < 360; deg ++)
	{
	    cout << progress.set(deg);

	    for (int x = -1; x < 2; x++)
		for (int y = -1; y < 2; y++)
		    for (int z = -1; z < 2; z++)
			if (x != 0 || y != 0 || z != 0)
			{
			    alpha = deg DEGREES;
			    rotVec = Vec3D(double(x), double(y), double(z));
			    rot = Rot3D(rotVec,alpha);
			    lin = Lin3D(rot);
			    
			    outRot = inverseCopy(rot) * vecRot * rot;
			    vec1 = outRot.getVec3D();
			    vec2 = vec * lin;
			    
			    
			    if (! isTiny(squaredLength(vec1 + vec2)))
			    {
				cout << vec1 << " !- " << vec2 << "\n";
				rotationTest.setFailed();
			    }
			}

	}
    }
}
