// ===========================================================================
//  Copyright 1994 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       IVectorN
//  TYPE:       c++ code
//  PROJECT:    VEGA Math Library
//  CONTENT:    vector and matrix of ints
//  VERSION:    0.6.0
// ===========================================================================
//  AUTHORS:    rft     Robert F. Tobler
// ===========================================================================
//  HISTORY:
//
//  30-sep-94 14:06:28  rft     last modification
//  28-jul-94 15:24:07  rft     created
// ===========================================================================

#include "VEGA/Math/Math.h"

#define IVECTORN1_NO_INLINE
#define IVECTORN2_NO_INLINE

#include "VEGA/Math/IVectorN.h"

#define INLINE_GLOBAL
#define INLINE_MEMBER

#include "VEGA/Math/IVectorN1.I"
#include "VEGA/Math/IVectorN2.I"

IVectorN::~IVectorN()
{
    kill();
}

SubIVectorN::~SubIVectorN()
{
    valuePtr = NIL(int *);
}

IMatrixNM::~IMatrixNM()
{
    kill();
}

SubIMatrixNM::~SubIMatrixNM()
{
    valuePtr = NIL(int *);
}

IVectorN IVectorN::bitReverseIndexedCopy() const
{
    int n = getLength();
    IVectorN result(n);

    if (n == 0 || (n & (n-1)))
    {
	EH_ERROR(IVectorN_errors, ERROR_VECTOR_NOT_POWER_OF_2,
		 "vector length not a power of 2: " << n);
	return result;
    }

    int highestBit = n >> 1;
    int secondHighestBit = highestBit >> 1;
    int thirdHighestBit = secondHighestBit >> 1;
    int index = 0;
    int reversed = 0;


    const int * vectorPtr = getLowerPtr();
    int * resultPtr = result.getLowerPtr();

    while (index < n)
    {
	resultPtr[reversed] = *vectorPtr++;
    
	index++;
	reversed ^= highestBit;
	if ((index & 1) == 0)
	{
	    reversed ^= secondHighestBit;
	    int reverseBit = thirdHighestBit;
	    for (int bit = 2; (index & bit) == 0; bit <<= 1)
	    {
		reversed ^= reverseBit; reverseBit >>= 1;
	    }
	}
    }
    return result;
}

IVectorN & IVectorN::bitReverseIndex()
{
    int n = getLength();
    
    if (n == 0 || (n & (n-1)))
    {
	EH_ERROR(IVectorN_errors, ERROR_VECTOR_NOT_POWER_OF_2,
		 "vector length not a power of 2: " << n);
	return *this;
    }

    int highestBit = n >> 1;
    int secondHighestBit = highestBit >> 1;
    int thirdHighestBit = secondHighestBit >> 1;
    int index = 0;
    int reversed = 0;

    int * v = getLowerPtr();

    int help;

    while (index < n)
    {
	if (index < reversed)
	{
	    help = v[reversed]; v[reversed] = v[index]; v[index] = help;
	}
	index++;
	reversed ^= highestBit;
	if ((index & 1) == 0)
	{
	    reversed ^= secondHighestBit;
	    int reverseBit = thirdHighestBit;
	    for (int bit = 2; (index & bit) == 0; bit <<= 1)
	    {
		reversed ^= reverseBit; reverseBit >>= 1;
	    }
	}
    }
    return *this;
}


// ===========================================================================
