// ===========================================================================
//  Copyright 1994 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       testCrd3
//  TYPE:       c++ code
//  PROJECT:    VEGA Math Library
//  CONTENT:    test module Crd3
//  VERSION:    1.0.0
// ===========================================================================
//  AUTHORS:    rft     Robert F. Tobler
// ===========================================================================
//  HISTORY:
//
//  30-sep-94 14:07:19  rft     last modification
//  30-sep-94 14:07:19  rft     created
// ===========================================================================

#include <VEGA/Base/Test.h>
#include <VEGA/Math/Math.h>
#include <VEGA/Math/Crd3.h>

void testCrd3(TestSuite & suite)
{
    TestModule module(suite, "Crd3");

    // ------------------------------------------------------- Singular Matrix
    {   
        Test test(module, "Singular Matrix", Test::failed);

        Crd3_errors.setState(ErrorHandler::useErrorNumber);
    
        Mat3 matrix(1,2,3,4,5,6,7,8,9);
                
        IF_EH_ERROR_IN(matrix.invert())
        {
            IF_EH_SWITCH(Crd3_errors)
            {
                case ERROR_CRD3_SINGULAR_MATRIX:
                    test.setPassed();
            }
        }
    }

    // ------------------------------------------------------ Matrix Inversion
    {
        Test test(module, "Matrix Inversion", Test::failed);
 
	Crd3_errors.setState(ErrorHandler::errorAndExit);

        Mat3 mat(1,1,1,1,1,2,1,2,2);
        Mat3 inv = inverseCopy(mat);
        
        Crd3 crd1(1,2,3);
        Crd3 crd2 = crd1 * mat;
        Crd3 crd3 = crd2 * inv;
        
        if (isTiny(squaredLength(crd3 - crd1)))
            test.setPassed();
    }
}