// ===========================================================================
//  Copyright 1994 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       testSimplePolynomial
//  TYPE:       c++ code
//  PROJECT:    VEGA Math Library
//  CONTENT:    test module SimplePolynomial
//  VERSION:    1.0.0
// ===========================================================================
//  AUTHORS:    rft     Robert F. Tobler
// ===========================================================================
//  HISTORY:
//
//  30-sep-94 14:07:19  rft     last modification
//  30-sep-94 14:07:19  rft     created
// ===========================================================================

#include <iostream.h>

#include <VEGA/Base/Bool.h>
#include <VEGA/Base/Test.h>
#include <VEGA/Math/Math.h>
#include <VEGA/Math/SimplePolynomial.h>

void testSimplePolynomial(TestSuite & suite)
{
    TestModule module(suite, "SimplePolynomial");

    // -------------------------------------------------- Unique Roots Order 4
    {   
        Test test(module, "Unique Roots Order 4", Test::passed);

	const int v = 9;

	const double values[] =
	{
	    -1.0,
	    0.0,
	    MATH_1_DIV_GOLDEN,
	    MATH_LN_2,
	    MATH_SQRT_2,
	    MATH_GOLDEN,
	    MATH_SQRT_3,
	    MATH_E,
	    MATH_PI
	};
	
	double poly[5];
	double polyroots[4];
	double roots[4];

	double a, b, c, d, diff;

	for (int i1 = 0; i1 < v; i1++)
	{
	    a =  polyroots[0] = values[i1];
	    for (int i2 = i1+1; i2 < v; i2++)
	    {
		b = polyroots[1] = values[i2];
		for (int i3 = i2+1; i3 < v; i3++)
		{
		    c = polyroots[2] = values[i3];
		    for (int i4 = i3+1; i4 < v; i4++)
		    {
			d = polyroots[3] = values[i4];
			{
			    poly[0] = a * b * c * d;
			    poly[1] = -  a * b * c - a * b * d
					- a * c * d - b * c * d;
			    poly[2] = a * b + a * c + b * c +
					a * d + b * d + c * d;
			    poly[3] = -a - b - c - d;
			    poly[4] = 1;

			    int n = SimplePolynomial::
					uniqueRootsO4(poly,roots);

			    if (n != 4)
				test.setFailed();
			    for (int k = 0; k < n; k++)
			    {
				diff = square(roots[k] - polyroots[k]);
				if (! isTiny(diff))
				{
				    cout << roots[k] << " != " <<
				    	polyroots[k] << ":" << diff << "\n";
				    test.setFailed();
				}
			    }
			}
		    }
		}
	    }
	}	
    }
}