// ===========================================================================
//  Copyright 1994 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       testVectorN
//  TYPE:       c++ code
//  PROJECT:    VEGA Math Library
//  CONTENT:    test module VectorN
//  VERSION:    1.0.0
// ===========================================================================
//  AUTHORS:    rft     Robert F. Tobler
// ===========================================================================
//  HISTORY:
//
//  30-sep-94 14:07:19  rft     last modification
//  30-sep-94 14:07:19  rft     created
// ===========================================================================

#include <iostream.h>
#include <iomanip.h>

#include <VEGA/Base/Bool.h>
#include <VEGA/Base/Test.h>
#include <VEGA/Math/Math.h>
#include <VEGA/Math/VectorN.h>

void testVectorN(TestSuite & suite)
{
    TestModule module(suite, "VectorN");

    int vecSize = 100;
    int matSize = 20;

    VectorN vector = VectorN(vecSize);
    VectorN vecOne = VectorN(vecSize);

    int i, j;

    {
	Test vectorAdd(module,"vector add", Test::passed);
    
	for (i = 0; i < vecSize; i++)
	{
	    vector[i] = 0;
	    vecOne[i] = 1;
	}
    
	for (i = 0; i < vecSize; i++)
	{
	    SubVectorN subVec = SubVectorN(vector,i,vecSize-i);
	    SubVectorN subOne = SubVectorN(vecOne,i,vecSize-i);
	    subVec += subOne;
	}
	
	for (i = 0; i < vecSize; i++)
	{
	    if (vector[i] != i + 1)
		vectorAdd.setFailed();
	}
    }

    {
	Test vectorSub(module,"vector sub", Test::passed);
    
	vector -= vecOne;
    
	for (i = 0; i < vecSize; i++)
	{
	    if (vector[i] != i)
		vectorSub.setFailed();
	}
    }

    {
	Test vectorMul(module,"vector mul", Test::passed);
    
	vector *= 2;
    
	for (i = 0; i < vecSize; i++)
	{
	    if (vector[i] != 2 * i)
		vectorMul.setFailed();
	}
    }

    MatrixNM matrix = MatrixNM(matSize,matSize);
    MatrixNM matOne = MatrixNM(matSize,matSize);

    {
	Test matrixAdd(module,"matrix add", Test::passed);
    
	for (i = 0; i < matSize; i++)
	    for (j = 0; j < matSize; j++)
	    {
		matrix[i][j] = 0;
		matOne[i][j] = 1;
	    }
    
	for (i = 0; i < matSize; i++)
	{
	    SubMatrixNM subMat = SubMatrixNM(matrix,0,i,matSize,matSize-i);
	    SubMatrixNM subOne = SubMatrixNM(matOne,0,i,matSize,matSize-i);
	    subMat += subOne;
	}
	
	for (i = 0; i < matSize; i++)
	{
	    SubMatrixNM subMat = SubMatrixNM(matrix,i,0,matSize-i,matSize);
	    SubMatrixNM subOne = SubMatrixNM(matOne,i,0,matSize-i,matSize);
	    subMat += subOne;
	}
	
	for (i = 0; i < matSize; i++)
	    for (j = 0; j < matSize; j++)
	    {
		if (matrix[i][j] != i + j + 2)
		    matrixAdd.setFailed();
	    }
    }
    
    {
	Test rectangleTest(module,"hartley transform rectangle", Test::passed);
	
	int n = 256;
	
	VectorN rectangle(n);
	
	for (i = 0; i < n; i++)
	{
	    if ( i < n/2 )
		rectangle[i] = 1.0;
	    else 
	        rectangle[i] = 0.0;
	}

	VectorN hartleyRectangle = rectangle.hartleyTransformedCopy();
	
	hartleyRectangle *= 1.0 / n;
	
	VectorN twiceRectangle = hartleyRectangle.hartleyTransformedCopy();
	
	for (i = 0; i < n; i++)
	{
	    if (!isTiny(square(rectangle[i] - twiceRectangle[i])))
		rectangleTest.setFailed();
	}
	
    }
}