// ===========================================================================
//  Copyright 1994 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       OutTools
//  TYPE:       c++ header
//  PROJECT:    VEGA Misc Library
//  CONTENT:    tools for generating stream output
//  VERSION:    1.0.0
// ===========================================================================
//  AUTHORS:    rft     Robert F. Tobler
// ===========================================================================
//  HISTORY:
//
//  30-sep-94 13:46:16  rft     last modification
//  16-Mar-93           rft     created
// ===========================================================================
//  MANUAL:
//
//  DESCRIPTION
//      This module contains tools for easily generating stream output.
//
//      The OutProgress class can be used to output a progress indicator. It
//      has to be initialized with the range which will be converted into
//      a progress indication between 0 and 100 percent. Each call to the
//      member function set with a new value between 0 and range will update
//      the percentage to be output. When an OutProgress object is streamed to
//      an outstream it will print the current percentage (only if it is
//      different from the last percentage that was output), move the cursor
//      back where it was before and then flush the outstream.
//
//  EXAMPLES
//      An OutProgress object can be used like this:
//
//              OutProgress progress(360);
//
//              for (int i = 0; i < 360; i++)
//              {
//                  cout << progress.set(i);
//
//                  // and now do the work.
//              }
//
//  SEE ALSO
//      VEGA, MiscLibrary
// ===========================================================================
#ifndef _MISC_OUTTOOLS_H_
#define _MISC_OUTTOOLS_H_

#include <iostream.h>
#include <iomanip.h>

#include "VEGA/Misc/Globals.h"

#ifdef __GNUC__
#pragma interface
#endif

extern "C++"
{

// ==================================================== tools for system types

class OutProgress
{
public:

    OutProgress(long newRange);
    
    OutProgress & set(long newValue);

    friend ostream & operator<<(ostream & s, OutProgress & p);
    
private:
    long range;
    long nowPercent;
    long outPercent;
};

} // extern "C++"

#ifndef OUTTOOLS_NO_INLINE

#define INLINE_GLOBAL COMPILER_INLINE_GLOBAL
#define INLINE_MEMBER COMPILER_INLINE_MEMBER

#include "VEGA/Misc/OutTools.I"

#undef INLINE_GLOBAL
#undef INLINE_MEMBER

#endif

#endif // _MISC_OUTTOOLS_H_
// ===========================================================================
