// ===========================================================================
//  Copyright 1994 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       TTools
//  TYPE:       c++ inline
//  PROJECT:    VEGA Misc Library
//  CONTENT:    template tools
//  VERSION:    1.0.0
// ===========================================================================
//  AUTHORS:    rft     Robert F. Tobler
// ===========================================================================
//  HISTORY:
//
//  30-sep-94 13:46:16  rft     last modification
//  16-Mar-93           rft     created
// ===========================================================================

extern "C++"
{

template<class T>
INLINE_TEMPLATE void copy(
        const T* srcPtr,
        int number,
        T* dstPtr
        )
{
    T* endPtr = dstPtr + number;
    while (dstPtr < endPtr)
    {
        *dstPtr++ = *srcPtr++;
    };
};

template<class T>
INLINE_TEMPLATE void copyDuff(
        const T* srcPtr,
        int number,
        T* dstPtr
        )
{
    if (number <= 0) return;

    register long n = (number + 15) / 16;       // number > 0 assumed

    switch (number % 16)
    {
        case 0: do {    *dstPtr++ = *srcPtr++;
        case 15:        *dstPtr++ = *srcPtr++;
        case 14:        *dstPtr++ = *srcPtr++;
        case 13:        *dstPtr++ = *srcPtr++;
        case 12:        *dstPtr++ = *srcPtr++;
        case 11:        *dstPtr++ = *srcPtr++;
        case 10:        *dstPtr++ = *srcPtr++;
        case 9:         *dstPtr++ = *srcPtr++;
        case 8:         *dstPtr++ = *srcPtr++;
        case 7:         *dstPtr++ = *srcPtr++;
        case 6:         *dstPtr++ = *srcPtr++;
        case 5:         *dstPtr++ = *srcPtr++;
        case 4:         *dstPtr++ = *srcPtr++;
        case 3:         *dstPtr++ = *srcPtr++;
        case 2:         *dstPtr++ = *srcPtr++;
        case 1:         *dstPtr++ = *srcPtr++;
                } while (--n);
    }
};

template<class T>
INLINE_TEMPLATE void copyAndPermute(
        const T* srcPtr,
        int number,
	const int * permutation,
        T* dstPtr
        )
{
    for (int i = 0; i < number; i++)
    {
	dstPtr[i] = srcPtr[permutation[i]];
    };
}

template<class T>
INLINE_TEMPLATE int isEqual(
        const T* onePtr,
        int number,
        const T* twoPtr
        )
{
    if (number > 0) do
    {
        if (*onePtr++ != *twoPtr++)
            return 0;
    }
    while (--number);
    return 1;
};

template<class T>
INLINE_TEMPLATE int isEqualDuff(
        const T* onePtr,
        long number,
        const T* twoPtr
        )
{
    if (number <= 0) return 1;

    register long n = (number + 15) / 16;       // number > 0 assumed

    switch (number % 16)
    {
        case 0: do {    if (*onePtr++ != *twoPtr++) return 0;
        case 15:        if (*onePtr++ != *twoPtr++) return 0;
        case 14:        if (*onePtr++ != *twoPtr++) return 0;
        case 13:        if (*onePtr++ != *twoPtr++) return 0;
        case 12:        if (*onePtr++ != *twoPtr++) return 0;
        case 11:        if (*onePtr++ != *twoPtr++) return 0;
        case 10:        if (*onePtr++ != *twoPtr++) return 0;
        case 9:         if (*onePtr++ != *twoPtr++) return 0;
        case 8:         if (*onePtr++ != *twoPtr++) return 0;
        case 7:         if (*onePtr++ != *twoPtr++) return 0;
        case 6:         if (*onePtr++ != *twoPtr++) return 0;
        case 5:         if (*onePtr++ != *twoPtr++) return 0;
        case 4:         if (*onePtr++ != *twoPtr++) return 0;
        case 3:         if (*onePtr++ != *twoPtr++) return 0;
        case 2:         if (*onePtr++ != *twoPtr++) return 0;
        case 1:         if (*onePtr++ != *twoPtr++) return 0;
                } while (--n);
    }
    return 1;
};

template<class T>
INLINE_TEMPLATE void swap(
        T* onePtr,
        long number,
        T* twoPtr
        )
{
    T help;
    if (number > 0) do
    {
        help = *onePtr; *onePtr++ = *twoPtr; *twoPtr++ = help;
    }
    while (--number);
};

template<class T>
INLINE_TEMPLATE void swapDuff(
        T* onePtr,
        long number,
        T* twoPtr
        )
{
    if (number <= 0) return;

    T h00, h01, h02, h03, h04, h05, h06, h07,
      h08, h09, h10, h11, h12, h13, h14, h15;

    register long n = (number + 15) / 16;       // number > 0 assumed

    switch (number % 16)
    {
        case 0: do {    h00 = *onePtr; *onePtr++ = *twoPtr; *twoPtr++ = h00;
        case 15:        h15 = *onePtr; *onePtr++ = *twoPtr; *twoPtr++ = h15;
        case 14:        h14 = *onePtr; *onePtr++ = *twoPtr; *twoPtr++ = h14;
        case 13:        h13 = *onePtr; *onePtr++ = *twoPtr; *twoPtr++ = h13;
        case 12:        h12 = *onePtr; *onePtr++ = *twoPtr; *twoPtr++ = h12;
        case 11:        h11 = *onePtr; *onePtr++ = *twoPtr; *twoPtr++ = h11;
        case 10:        h10 = *onePtr; *onePtr++ = *twoPtr; *twoPtr++ = h10;
        case 9:         h09 = *onePtr; *onePtr++ = *twoPtr; *twoPtr++ = h09;
        case 8:         h08 = *onePtr; *onePtr++ = *twoPtr; *twoPtr++ = h08;
        case 7:         h07 = *onePtr; *onePtr++ = *twoPtr; *twoPtr++ = h07;
        case 6:         h06 = *onePtr; *onePtr++ = *twoPtr; *twoPtr++ = h06;
        case 5:         h05 = *onePtr; *onePtr++ = *twoPtr; *twoPtr++ = h05;
        case 4:         h04 = *onePtr; *onePtr++ = *twoPtr; *twoPtr++ = h04;
        case 3:         h03 = *onePtr; *onePtr++ = *twoPtr; *twoPtr++ = h03;
        case 2:         h02 = *onePtr; *onePtr++ = *twoPtr; *twoPtr++ = h02;
        case 1:         h01 = *onePtr; *onePtr++ = *twoPtr; *twoPtr++ = h01;
                } while (--n);
    }
};

template<class T>
INLINE_TEMPLATE void swap(
        T& a,
        T& b
        )
{
    T tmp = a; a = b; b = tmb;
};

template<class T>
INLINE_TEMPLATE void sort3ascending(
        T* array
        )
{
    if (array[0] > array[1])
    {
        if (array[0] > array[2])
        {                                       // array[0] is biggest
            T help = array[2];
            array[2] = array[0];
            if (array[1] > help)
            {
                array[0] = help;
            }
            else
            {
                array[0] = array[1];
                array[1] = help;
            }
        }
        else
        {
            T help = array[1];
    
            array[1] = array[0];
            array[0] = help;
        }
    }
    else
    {
        if (array[1] > array[2])
        {                                       // array[1] is biggest
            T help = array[2];
            array[2] = array[1];
            if (array[0] > help)
            {
                array[1] = array[0];
                array[0] = help;
            }
            else
                array[1] = help;
        }
    }
};

template<class T>
INLINE_TEMPLATE int mergeAscending(
        const T * ptr1,
        int n1,
        const T * ptr2,
        int n2,
        T * ptr
        )
{
    int n = n1 + n2;

    while (n1 > 0 && n2 > 0)
    {
        if (*ptr1 < *ptr2)
        {
            *ptr++ = *ptr1++;
            n1--;
        }
        else
        {
            *ptr++ = *ptr2++;
            n2--;
        }
    }
    while (n1 > 0)
    {
        *ptr++ = *ptr1++;
        n1--;
    }
    while (n2 > 0)
    {
        *ptr++ = *ptr2++;
        n2--;
    }
    return (n);
};

} // extern "C++"

// ===========================================================================
