// ===========================================================================
//  Copyright 1994 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       Math
//  TYPE:       c++ inline
//  PROJECT:    VEGA Math Library
//  CONTENT:    mathematical functions
//  VERSION:    1.0.0
// ===========================================================================
//  AUTHORS:    rft     Robert F. Tobler
// ===========================================================================
//  HISTORY:
//
//  30-sep-94 14:06:28  rft     last modification
//  28-Aug-93           rft     created
// ===========================================================================

extern "C++"
{

#ifdef SOFTMATH_ACOS_ASIN_ATAN
#define acos(x)         SoftMath::acos(x)
#define asin(x)         SoftMath::asin(x)
#define atan(x)         SoftMath::atan(x)
#define atan2(y,x)      SoftMath::atan2(y,x)
#endif

#ifdef SOFTMATH_COS_SIN_TAN
#define cos(x)          SoftMath::cos(x)
#define sin(x)          SoftMath::sin(x)
#define tan(x)          SoftMath::tan(x)
#endif

#ifdef SOFTMATH_COSH_SINH_TANH
#define cosh(x)         SoftMath::cosh(x)
#define sinh(x)         SoftMath::sinh(x)
#define tanh(x)         SoftMath::tanh(x)
#endif

#ifdef SOFTMATH_EXP_LOG_POW
#define exp(x)          SoftMath::exp(x)
#define log(x)          SoftMath::log(x)
#define log10(x)        SoftMath::log10(x)
#define pow(x,y)        SoftMath::pow(x,y)
#endif

#ifdef SOFTMATH_CEIL_FLOOR_FMOD_FABS
#define ceil(x)         (-SoftMath::floor(-(x)))
#define floor(x)        SoftMath::floor(x)
#define fmod(x,y)       SoftMath::fmod(x,y)
#define fabs(x)         SoftMath::fabs(x)
#endif


INLINE_GLOBAL int sign(float x)
{
    if ( x > 0.0 )
        return 1;
    else if ( x < 0.0)
        return -1;
    else
        return 0;
}

INLINE_GLOBAL int sign(double x)
{
    if ( x > 0.0 )
        return 1;
    else if ( x < 0.0)
        return -1;
    else
        return 0;
}

INLINE_GLOBAL float  abs(float  a) { return (a >= 0 ? a : -a); }
INLINE_GLOBAL double abs(double a) { return (a >= 0 ? a : -a); }
INLINE_GLOBAL float  max(float  a, float  b) { return (a >= b ? a : b); }
INLINE_GLOBAL double max(double a, double b) { return (a >= b ? a : b); }
INLINE_GLOBAL float  min(float  a, float  b) { return (a <= b ? a : b); }
INLINE_GLOBAL double min(double a, double b) { return (a <= b ? a : b); }

INLINE_GLOBAL float square(float x)    { return (x * x); }
INLINE_GLOBAL float cube(float x)      { return (x * x * x); }
INLINE_GLOBAL double square(double x)  { return (x * x); }
INLINE_GLOBAL double cube(double x)    { return (x * x * x); }
INLINE_GLOBAL double round(double x)   { return floor(x + 0.5); }
INLINE_GLOBAL double fract(double x)   { return (x - floor(x)); }

INLINE_GLOBAL int isTiny(float x)      { return (fabs(x) < FLOAT_TINY); }
INLINE_GLOBAL int isHuge(float x)      { return (fabs(x) > FLOAT_HUGE); }
INLINE_GLOBAL int isTiny(double x)     { return (fabs(x) < DOUBLE_TINY); }
INLINE_GLOBAL int isHuge(double x)     { return (fabs(x) > DOUBLE_HUGE); }

#ifdef __BCPLUSPLUS__
INLINE_GLOBAL double cbrt(double x)	{ return (pow(x, MATH_1_DIV_3)); }
#endif

INLINE_GLOBAL float clamp(float x, float lo, float hi)
{
    if (x < lo)
	return lo;
    else if (x > hi)
	return hi;
    else
	return x;
}

INLINE_GLOBAL double clamp(double x, double lo, double hi)
{
    if (x < lo)
	return lo;
    else if (x > hi)
	return hi;
    else
	return x;
}

INLINE_GLOBAL double det2x2(double x00, double x01, double x10, double x11)
{
    return (x00 * x11 - x01 * x10);
}

INLINE_GLOBAL double det3x3(
        double x00, double x10, double x20,
        double x01, double x11, double x21,
        double x02, double x12, double x22
        )

{
    return (  x00 * det2x2(x11,x12,x21,x22)
            - x01 * det2x2(x10,x12,x20,x22)
            + x02 * det2x2(x10,x11,x20,x21));
}

INLINE_MEMBER double SoftMath::acos(double x)
{
    return (MATH_PI_DIV_2 - SoftMath::asin(x));
}

INLINE_MEMBER double SoftMath::atan(double x)
{
    return (SoftMath::atan2(x,1.0));
}

INLINE_MEMBER double SoftMath::cosh(double x)
{
    x = exp(x);
    return (0.5 * (x + 1.0/x));
}

INLINE_MEMBER double SoftMath::sinh(double x)
{
    x = exp(x);
    return (0.5 * (x - 1.0/x));
}

INLINE_MEMBER double SoftMath::tanh(double x)
{
    x = exp(2 * x);
    return ((x - 1.0)/(x + 1.0));
}

INLINE_MEMBER double SoftMath::log10(double x)
{
    return (SoftMath::log(x) * MATH_1_DIV_LN_10);
}

INLINE_MEMBER double SoftMath::pow(double x, double y)
{

    if (x == 0.0)
	return (0.0);         // Forget problems with pow(0,0)
    else if (y == 0.0)
	return (1.0);
    else
	return (SoftMath::exp(y * SoftMath::log(x)));
}

INLINE_MEMBER double SoftMath::fabs(double x)
{
    if (x >= 0.0)
        return (x);
    else
        return (-x);
}

} // extern "C++"

// ===========================================================================


