// ===========================================================================
//  Copyright 1994 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       LibraryTest
//  TYPE:       c++ code
//  PROJECT:    VEGA Math Library
//  CONTENT:    VEGA Math Library Test
//  VERSION:    1.0.0
// ===========================================================================
//  AUTHORS:    rft     Robert F. Tobler
// ===========================================================================
//  HISTORY:
//
//  30-sep-94 14:07:19  rft     last modification
//  30-sep-94 14:07:19  rft     created
// ===========================================================================

extern "C"
{
#include <string.h>
}

#include <VEGA/Base/library.h>
#include <VEGA/Math/globals.h>

void testCrd3(TestSuite & suite);
void testCrd4(TestSuite & suite);
void testSimplePolynomial(TestSuite & suite);
void testVectorN(TestSuite & suite);

Bool allTests = Bool::yes;

int doTest(const OptionString & option, const char * name)
{
    return (isTrue(allTests) || strcmp(option.getValue(), name) == 0);
}

int main(int argc, char * argv[])
{
    OptionString singleOpt("single",     "s");

    TR_CODE(OptionFlag debugOpt("debug",         "d"));
    TR_CODE(OptionFlag nCrd3c("noCrd3constructors", "nCrd3c"));
    TR_CODE(OptionFlag nCrd4c("noCrd4constructors", "nCrd4c"));

    Option::parse(argc,argv);

    TR_CODE(if (debugOpt.isSpecified()) TR_ON(VEGA_Math_tracers));
    TR_CODE(if (nCrd3c.isSpecified()) TR_OFF(Crd3_constructors));
    TR_CODE(if (nCrd4c.isSpecified()) TR_OFF(Crd4_constructors));

    TestSuite suite("VEGA Math Library");
    
    if (singleOpt.isSpecified()) allTests = Bool::no;
    
    if (doTest(singleOpt,"Crd3"))		testCrd3(suite);
    if (doTest(singleOpt,"Crd4"))		testCrd4(suite);
    if (doTest(singleOpt,"SimplePolynomial"))	testSimplePolynomial(suite);
    if (doTest(singleOpt,"Vector"))		testVectorN(suite);

    TR_CODE(if (debugOpt.isSpecified()) TR_OFF(VEGA_Math_tracers));

    return (0);
}
