// ===========================================================================
//  Copyright 1994 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       testCrd4
//  TYPE:       c++ code
//  PROJECT:    VEGA Math Library
//  CONTENT:    test module Crd4
//  VERSION:    1.0.0
// ===========================================================================
//  AUTHORS:    rft     Robert F. Tobler
// ===========================================================================
//  HISTORY:
//
//  30-sep-94 14:07:19  rft     last modification
//  30-sep-94 14:07:19  rft     created
// ===========================================================================

#include <VEGA/Base/test.h>
#include <VEGA/Math/math.h>
#include <VEGA/Math/crd4.h>


void testCrd4(TestSuite & suite)
{
    TestModule module(suite, "Crd4");

    // ------------------------------------------------------- Singular Matrix
    {   
        Test test(module, "Singular Matrix", Test::failed);

        Crd4_errors.setState(ErrorHandler::useErrorNumber);
    
        Mat4 matrix( 1, 2, 3, 4,
                     5, 6, 7, 8,
                     9,10,11,12,
                    13,14,15,16);
                
        IF_EH_ERROR_IN(matrix.invert())
        {
            IF_EH_SWITCH(Crd4_errors)
            {
                case ERROR_CRD4_SINGULAR_MATRIX:
                    test.setPassed();
            }
        }
    }

    // ------------------------------------------------------ Matrix Inversion
    {
        Test test(module, "Matrix Inversion", Test::failed);

        Crd4_errors.setState(ErrorHandler::errorAndExit);
        
        Mat4 mat(1,0,0,0,
                 0,2,0,0,
                 0,0,3,0,
                 0,0,0,4);

        Mat4 inv = inverseCopy(mat);

	Crd4 crd1(1,2,3,4);
	Crd4 crd2 = crd1 * mat;
	Crd4 crd3 = crd2 * inv;
	
	if (isTiny(squaredLength(crd3 - crd1)))
	    test.setPassed();
    }
}