// ===========================================================================
//  Copyright 1994 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       Bits
//  TYPE:       c++ inline
//  PROJECT:    VEGA Base Library
//  CONTENT:    bit manipulation routines
//  VERSION:    1.0.0
// ===========================================================================
//  AUTHORS:    rft     Robert F. Tobler
// ===========================================================================
//  HISTORY:
//
//  30-sep-94 13:46:16  rft     last modification
//  10-aug-94 11:52:39  rft     created
// ===========================================================================

extern "C++"
{

INLINE_GLOBAL int Bits::countByTable(UInt8 n)
{
    return countTable[n];
}

INLINE_GLOBAL int Bits::countByTable(UInt16 n)
{
    return countTable[n & 0xff] +
	   countTable[n >> 8];
}

INLINE_GLOBAL int Bits::countByTable(UInt32 n)
{
    int bits = countTable[n & 0xff];
    
    bits += countTable[(n >>= 8) & 0xff];
    bits += countTable[(n >>= 8) & 0xff];

    return bits + countTable[n >> 8];
}

INLINE_GLOBAL UInt8 Bits::reverseByTable(UInt8 n)
{
    return reverseTable[n];
}

INLINE_GLOBAL UInt16 Bits::reverseByTable(UInt16 n)
{
    return  (reverseTable[n & 0xff] << 8) |
	    reverseTable[n >> 8];
}

INLINE_GLOBAL UInt32 Bits::reverseByTable(UInt32 n)
{
    UInt32 reverse = reverseTable[n & 0xff];
    
    reverse = (reverse << 8) | reverseTable[(n >>= 8) & 0xff];
    reverse = (reverse << 8) | reverseTable[(n >>= 8) & 0xff];

    return (reverse << 8) | reverseTable[n >> 8];
}

INLINE_GLOBAL int Bits::countTricky(UInt32 n)
{
    n = n - ((n >> 1) & 033333333333) - ((n >> 2) & 011111111111);
    n = ((n + (n >> 3)) & 030707070707);
    n =  (n + (n >> 6));
    n = (n + (n >> 12) + (n >> 24)) & 077;	/* 6+3+3+5 = 16 ops */

    return (int)n;
}


//
// following regular algorithms based on macros written by
// Phil Howard pdh@netcom.com
//

INLINE_GLOBAL int Bits::countRegular(UInt8 n)
{
    n = ((n >> 1) & 0x55) + (n & 0x55);
    n = ((n >> 2) & 0x33) + (n & 0x33);
    n = ((n >> 4) & 0x07) + (n & 0x07);

    return (int)n;
}

INLINE_GLOBAL int Bits::countRegular(UInt16 n)
{
    n = ((n >> 1) & 0x5555) + (n & 0x5555);
    n = ((n >> 2) & 0x3333) + (n & 0x3333);
    n = ((n >> 4) & 0x0707) + (n & 0x0707);
    n = ((n >> 8) & 0x000f) + (n & 0x000f);

    return (int)n;
}

INLINE_GLOBAL int Bits::countRegular(UInt32 n)
{
    n = ((n >>  1) & 0x55555555) + (n & 0x55555555);
    n = ((n >>  2) & 0x33333333) + (n & 0x33333333);
    n = ((n >>  4) & 0x07070707) + (n & 0x07070707);
    n = ((n >>  8) & 0x000f000f) + (n & 0x000f000f);
    n = ((n >> 16) & 0x0000001f) + (n & 0x0000001f);

    return (int)n;
}

INLINE_GLOBAL UInt8 Bits::reverseRegular(UInt8 n)
{
    n = ((n >> 1) & 0x55) | ((n & 0x55) << 1);
    n = ((n >> 2) & 0x33) | ((n & 0x33) << 2);
    n = ((n >> 4) & 0x0f) | ((n & 0x0f) << 4);

    return n;
}

INLINE_GLOBAL UInt16 Bits::reverseRegular(UInt16 n)
{
    n = ((n >> 1) & 0x5555) | ((n & 0x5555) << 1);
    n = ((n >> 2) & 0x3333) | ((n & 0x3333) << 2);
    n = ((n >> 4) & 0x0f0f) | ((n & 0x0f0f) << 4);
    n = ((n >> 8) & 0x00ff) | ((n & 0x00ff) << 8);

    return n;
}

INLINE_GLOBAL UInt32 Bits::reverseRegular(UInt32 n)
{
    n = ((n >>  1) & 0x55555555) | ((n & 0x55555555) <<  1);
    n = ((n >>  2) & 0x33333333) | ((n & 0x33333333) <<  2);
    n = ((n >>  4) & 0x0f0f0f0f) | ((n & 0x0f0f0f0f) <<  4);
    n = ((n >>  8) & 0x00ff00ff) | ((n & 0x00ff00ff) <<  8);
    n = ((n >> 16) & 0x0000ffff) | ((n & 0x0000ffff) << 16);

    return n;
}

#ifdef LONGLONG

INLINE_GLOBAL int Bits::countByTable(UInt64 n)
{
    int bits = countTable[n & 0xff];
    
    bits += countTable[(n >>= 8) & 0xff];
    bits += countTable[(n >>= 8) & 0xff];
    bits += countTable[(n >>= 8) & 0xff];
    bits += countTable[(n >>= 8) & 0xff];
    bits += countTable[(n >>= 8) & 0xff];
    bits += countTable[(n >>= 8) & 0xff];

    return bits + countTable[n >> 8];
}

INLINE_GLOBAL UInt64 Bits::reverseByTable(UInt64 n)
{
    UInt64 reverse = reverseTable[n & 0xff];
    
    reverse = (reverse << 8) | reverseTable[(n >>= 8) & 0xff];
    reverse = (reverse << 8) | reverseTable[(n >>= 8) & 0xff];
    reverse = (reverse << 8) | reverseTable[(n >>= 8) & 0xff];
    reverse = (reverse << 8) | reverseTable[(n >>= 8) & 0xff];
    reverse = (reverse << 8) | reverseTable[(n >>= 8) & 0xff];
    reverse = (reverse << 8) | reverseTable[(n >>= 8) & 0xff];

    return (reverse << 8) | reverseTable[n >> 8];
}

INLINE_GLOBAL int Bits::countRegular(UInt64 n)
{
    n = ((n >>  1) & 0x5555555555555555ULL) + (n & 0x5555555555555555ULL);
    n = ((n >>  2) & 0x3333333333333333ULL) + (n & 0x3333333333333333ULL);
    n = ((n >>  4) & 0x0707070707070707ULL) + (n & 0x0707070707070707ULL);
    n = ((n >>  8) & 0x000f000f000f000fULL) + (n & 0x000f000f000f000fULL);
    n = ((n >> 16) & 0x0000001f0000001fULL) + (n & 0x0000001f0000001fULL);
    n = ((n >> 32) & 0x000000000000003fULL) + (n & 0x000000000000003fULL);

    return (int)n;
}

INLINE_GLOBAL UInt64 Bits::reverseRegular(UInt64 n)
{
    n = ((n >>  1) & 0x5555555555555555ULL) |
	       ((n & 0x5555555555555555ULL) <<  1);
    n = ((n >>  2) & 0x3333333333333333ULL) |
    	       ((n & 0x3333333333333333ULL) <<  2);
    n = ((n >>  4) & 0x0f0f0f0f0f0f0f0fULL) |
	       ((n & 0x0f0f0f0f0f0f0f0fULL) <<  4);
    n = ((n >>  8) & 0x00ff00ff00ff00ffULL) |
	       ((n & 0x00ff00ff00ff00ffULL) <<  8);
    n = ((n >> 16) & 0x0000ffff0000ffffULL) |
	       ((n & 0x0000ffff0000ffffULL) << 16);
    n = ((n >> 32) & 0x00000000ffffffffULL) |
	       ((n & 0x00000000ffffffffULL) << 32);

    return n;
}

#endif // LONGLONG

} // extern "C++"

// ===========================================================================

