#!/usr/local/bin/perl
## ===========================================================================
##  Copyright 1994 Technical University of Vienna. All Rights reserved.
## ===========================================================================
##  NAME:       sftool
##  TYPE:       perl script
##  PROJECT:    SF - Source File Tools
##  CONTENT:    tool routines
##  VERSION:    0.95.0
## ===========================================================================
##  AUTHORS:    rft     Robert F. Tobler
## ===========================================================================
##  HISTORY:
##
##  26-sep-94 16:54:35  rft     last modification
##  11-mar-94 19:33:12  rft     changed DESCRIPTION to MANUAL
##  03-nov-93           rft     created
## ===========================================================================
package sftool;

if ($main'debug) { $debug = 1; }

if ($main'system eq 'OS2')
{
    $sys_name = 'OS2';
    $sys_option = '[/-]';
    $sys_slash = '\\';
}
elsif ($main'system eq 'UNIX')
{
    $sys_name = 'UNIX';
    $sys_option = '-';
    $sys_slash = '/';
}
else
{
    die "system undefined - please contact tool administrator";
}

## ===========================================================================
##  various useful routines
## ===========================================================================

sub main'exit
{
    if ($sys_name ne 'OS2') { print "\n"; }
    exit;
}

sub main'fatal
{
    print "\nERROR: $_[0]!\n";
    &main'exit;
}

sub main'error
{
    print "\nERROR: $_[0]!\n";
}

sub main'warning
{
    print "\nWARNING: $_[0]!\n";
}

sub main'printselfdescription
{
    open(SELF,$0);
    while(<SELF>)
    {
        if (/^##  MANUAL:/o)
        {
            while(<SELF>)
            {
                if (/^## =+$/o) { last; }
                s/^##/  /o;
                print;
            }
            last;
        }
    }
}

sub main'untab8
{
    local ($_) = $_[0];
    while (/\t/o)
    {
	$_ = $`.(' ' x (8 - length($`) % 8)).$';
    }
    $_;
}
## =====================================================================
##  commandline option handling routines
## =====================================================================

sub main'argv_opt
{
    &argv_init(@_);
    $argv_type{$_[0]} = 1;
}

sub main'argv_par
{
    &argv_init(@_);
    $argv_type{$_[0]} = 2;
}

sub main'argv_optpar
{
    &argv_init(@_);
    $argv_type{$_[0]} = 3;
}

sub main'argv_seq
{
    &argv_init(@_);
    $argv_type{$_[0]} = 4;
}

sub argv_init
{
    local($long) = $_[0];
    local($short) = $_[1];
    
    if ($short)
    {
        if ($argv_long{$short})
        {
            die "short option name '$short' already in use";
        }        
        $argv_long{$short} = $long;
    }

    if ($argv_type{$cmd_long})
    {
        die "long option name '$cmd_long' already in use";
    }
}


sub main'argv_parse
{
    local($arg);
    local($arg_par) = 0;
    local($arg_long);
    local($arg_type);
    local($arg_name);

    print "ARGV_PARSE:\n" if ($debug);

    while ($_ = shift(@ARGV))
    {
        if (/^$sys_option/o)
        {
            if ($arg_par && $arg_type == 2)
	    {
		&main'fatal("option '$arg' needs parameter");
	    }
	    $arg_par = 0;
            $arg = $_;
            $_ = $';
            tr/A-Z/a-z/;
            if (/^$sys_option/o)
            {
                $arg_long = $';
            }
            else
            {
                $arg_long = $argv_long{$_};
            }
            $arg_type = $argv_type{$arg_long};
            if (! $arg_type)
            {
                &main'fatal("unknown option '$arg'");
            }
            $arg_name = "main'arg_".$arg_long;
            if (eval('$'.$arg_name.'++') > 0)
            {
                &main'fatal("'$arg' has already been specified");
            }
            if ($arg_type == 1)
            {
                print "  opt:      '$arg'\n" if ($debug);
            }
            elsif ($arg_type == 2)
            {
                print "  par:      '$arg'\n" if ($debug);
                $arg_par = 1;
            }
            elsif ($arg_type == 3)
            {
                print "  opt/par:  '$arg'\n" if ($debug);
		$arg_par = 1;
            }
            elsif ($arg_type == 4)
            {
                print "  seq:      '$arg'\n" if ($debug);
                $arg_par = -1;
            }
        }
        else
        {
            if ($arg_par != 0)
            {
                @list = eval('@'.$arg_name);
                push(@list,$_);
                eval('@'.$arg_name.'=@list;');
		$arg_par--;
                next;
            }
            push(@main'argvec,$_);
        }
    }
    if ($arg_par && $arg_type == 2)
    {
	&main'fatal("option '$arg' needs parameter");
    }
    print "\n" if ($debug);
}

## =====================================================================
##  file handling routines
## =====================================================================

sub main'file_rename
{
    local($source) = $_[0];
    local($target) = $_[1];
    if ($sys_name eq 'OS2')
    {
        `del $target`;
        `rename $source $target`;
    }
    else
    {
        rename($source,$target);
    }
}

sub main'file_copy
{
    local($infile) = $_[0];
    local($outfile) = $_[1];

    open(INFILE,"<$infile");
    open(OUTFILE, ">$outfile");

    while (<INFILE>)
    {
        print OUTFILE;
    }

    close OUTFILE;
    close INFILE;
}

## =====================================================================
##  filename handling routines
## =====================================================================

sub main'filename_getpath
{
    local($slash) = rindex($_[0],$sys_slash);
    if ($slash < 0)
    {
        '';
    }
    else
    {
        substr($_[0],0,$slash + 1);
    }
}

sub main'filename_splitpath
{
    local($slash) = rindex($_[0],$sys_slash);
    if ($slash < 0)
    {
        ('',$_[0]);
    }
    else
    {
        (substr($_[0],0,$slash + 1),
         substr($_[0],$slash + 1, length($_[0]) - $slash - 1));
    }
}

sub main'filename_split
{
    local($point) = rindex($_[0],'.');
    local($slash) = rindex($_[0],$sys_slash);
    if ($point <= $slash)
    {
        ($_[0],'.');
    }
    else
    {
        (substr($_[0],0,$point),
         substr($_[0],$point + 1, length($_[0]) - $point - 1));
    }
}

sub main'filename_join
{
    local($name) = $_[0];
    local($ext) = $_[1];
    if ($ext eq '.')
    {
        $name;
    }
    else
    {
        $name.'.'.$ext;
    }
}


sub main'filename_getext
{
    local($point) = rindex($_[0],'.');
    local($slash) = rindex($_[0],$sys_slash);
    if ($point <= $slash)
    {
        '.';
    }
    else
    {
        substr($_[0],$point + 1, length($_[0]) - $point - 1);
    }
}

sub main'filename_getname
{
    local($point) = rindex($_[0],'.');
    local($slash) = rindex($_[0],$sys_slash);
    if ($point <= $slash)
    {
        $_[0];
    }
    else
    {
        if ($slash < 0)
        {
            substr($_[0],0,$point);
        }
        else
        {
            substr($_[0],$slash + 1, $point - $slash - 1);
        }
    }
}

sub main'filename_getnameext
{
    local($slash) = rindex($_[0],$sys_slash);
    if ($slash < 0)
    {
        $_[0];
    }
    else
    {
	substr($_[0],$slash + 1);
    }
}


sub main'filename_getdir
{
    local($slash) = rindex($_[0],$sys_slash);
    if ($slash < 0)
    {
        ();
    }
    else
    {
        substr($_[0],0,$slash);
    }
}


## =====================================================================
##  filetype handling routines
## =====================================================================

sub filetypeasc
{
    local($type) = $_[0];
    local($ext);

    $filetype_ascii{$type} = 1;
    $filetype_clt{$type} = $_[1];
    $filetype_crt{$type} = $_[2];
    $filetype_extoftyp{$type} = $_[3];

    foreach $ext (split(/:/,$_[3]))
    {
        $filetype_typofext{$ext} =
            &colonlist_push($filetype_typofext{$ext},$type);
    }
    
    push(@filetypes,$type);
}

sub filetypebin
{
    local($type) = $_[0];
    local($ext);

    $filetype_ascii{$type} = 0;
    $filetype_extoftyp{$type} = $_[1];

    foreach $ext (split(/:/,$_[1]))
    {
        $filetype_typofext{$ext} =
            &colonlist_push($filetype_typofext{$ext},$type);
    }
    
    push(@filetypes,$type);
}

sub main'filetype_init
{
    local($type);
    local($ext);

    if ($sys_name eq 'OS2') { $perl_ext = ':cmd'; }

    ## -----------------------------------------------------------------------
    ##  There are a few ways to make these routines recognize filetypes based
    ##  on the filename:
    ##
    ##  <ext>           only the extension is significant
    ##  <name>.         only the name without extension is significant
    ##  <name>.<ext>    name and extension are significant
    ##  .<ext>          matches only a file starting with a dot and given ext.
    ##  <name>..        matches only a file without extension and given name
    ## -----------------------------------------------------------------------


    &filetypeasc('bookmaster script',	'.* ', '',	'scr');
    &filetypeasc('bash script',		'## ', '',	'.:bash');
    &filetypebin('bcc project',		'prj');
    &filetypeasc('bison source',	'/* ', ' */',	'y');
    &filetypeasc('c code',		'/* ', ' */',	'c');
    &filetypeasc('c header',		'/* ', ' */',	'h');

    &filetypeasc('c++ code',		'// ', '',
					'c++:cc:C:cpp:cxx');
    &filetypeasc('c++ header',		'// ', '',
					'h:h++:hh:H:hpp:hxx');
    &filetypeasc('c++ inline',		'// ', '',
					'i++:ii:I:ipp:ixx:inl');

    &filetypeasc('csh script',		'## ', '',	'.:csh');
    &filetypeasc('dos batch',		'rem', '',	'.bat');
    &filetypeasc('flex source',		'/* ', ' */',	'l');
    &filetypeasc('lex source',		'/* ', ' */',	'l');

    &filetypeasc('makefile',		'## ', '',
                                        'makefile.:Makefile.:mak');

    &filetypeasc('os2 cmd',		'rem',  '',	'cmd');
    &filetypeasc('perl script',		'## ', '',	'.:pl'.$perl_ext);
    &filetypeasc('python script',	'## ', '',	'.:py');
    &filetypeasc('rexx cmd',		'/* ', ' */',	'cmd');
    &filetypeasc('sfhead config',	'## ', '',	'.sfhead:_sfhead_..');
    &filetypeasc('sfname config',	'## ', '',	'.sfname:_sfname_..');
    &filetypeasc('sh script',		'## ', '',	'.:sh');
    &filetypeasc('tcsh script',		'## ', '',	'.:tcsh');
    &filetypeasc('yacc source',		'/* ', ' */',	'y');
    &filetypeasc('zsh script',		'## ', '',	'.:zsh');

    ## --------------------------------------------------------- include types

    $incltype{'bookmaster script'} =    'bookmaster';
    $incltype{'bison source'} =		'c';
    $incltype{'c code'} =               'c';
    $incltype{'c header'} =             'c';
    $incltype{'c++ code'} =             'c';
    $incltype{'c++ header'} =           'c';
    $incltype{'c++ inline'} =           'c';
    $incltype{'flex source'} =		'c';
    $incltype{'lex source'} =           'c';
    $incltype{'yacc source'} =          'c';
    $incltype{'sfname config'} =	'c';

    $inclline{'bookmaster'} =   '^(\.im\s+)(\S+)(.*)$';
    $inclline{'c'} =            '^(\s*#\s*include\s+["\<])([^"\>]*)(["\>].*)$';

    *main'filetypes = *filetypes;
    *main'filetype_clt = *filetype_clt;
    *main'filetype_crt = *filetype_crt;
    *main'filetype_extoftyp = *filetype_extoftyp;
    *main'filetype_typofext = *filetype_typofext;
    *main'incltype = *incltype;
    *main'inclline = *inclline;
}

sub colonlist_push
{
    if ($_[0])
        { $_[0].':'.$_[1]; }
    else
        { $_[1]; }
}

sub main'filetype_isascii
{
    $filetype_ascii{$_[0]};
}

sub main'filetype_getbyname
{
    local($file) = $_[0];
    local($ask) = $_[1];
    local($ext) = &main'filename_getext($file);
    local($input);

    print("FILETYPE_GETBYNAME:\n") if $debug;

    if ($system eq 'OS2')
    {
        $extension =~ tr/A-Z/a-z/;
    }

    $type = $filetype_typofext{&main'filename_getname($file).'.'.$ext};

    if (! $type)
    {
        $type = $filetype_typofext{&main'filename_getname($file).'.'};
    }
    
    if (! $type)
    {
        $type = $filetype_typofext{$ext};
    }

    if ($type =~ /:/o)
    {
        if ($ask == 1)
        {
            $number = 0;
            while ($number != 1)
            {
                print "    TYPE($type): ";
                $input = <STDIN>;
                chop($input);
    
                if ($input =~ /:/o) { next; }
                @types = &arraymatch($input,split(/:/,$type));
                $number = @types;
            }
            $type = $types[0];
        }
	elsif ($ask == -1)
	{
	    $type = $`;
	}
        else
        {
            $type = '';
        }
    }

    print "type: $type\n\n" if ($debug);
    $type;
}

sub arraymatch
{
    local($pattern,@array) = @_;
    local($element);
    local(@return);

    foreach $element (@array)
    {
        if (substr($element,0,length($pattern)) eq $pattern)
        {
            push(@return,$element);
        }
    }

    @return;
}

sub main'filetype_byheader
{
    local($filename) = $_[0];
    local($type);
    local($knowntype);

    print("FILETYPE_BYHEADER:\n") if $debug;

    if ($filename)
    {
        open(TESTFILE,$filename);
        print "file: '$filename'\n" if ($debug);
    }
    else
    {
        *TESTFILE = main'INFILE;
        print "file: INFILE\n" if ($debug);
    }

    for ($i = 0; $i < 16; $i++)
    {
        $_ = <TESTFILE>;
        
        print ("line: $_") if ($debug);
        
        if (/^...=+...$/o)
        {
            $main'filetype_linelength = length($_) - 1;
        }
        elsif (/^... TYPE:\s+(.*)$/o)
        {
            foreach $knowntype (@filetypes)
            {
                if (substr($1,0,length($knowntype)) eq $knowntype)
                {
                    $type = $knowntype;
                }
            }
            last;
        }
    }
    if ($filename)
    {
        close(TESTFILE);
    }

    print "type: $type\n\n" if ($debug);

    $type;
}

