// ===========================================================================
//  Copyright 1994 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       VEGA
//  TYPE:       c++ header
//  PROJECT:    VEGA - Vienna Environment for Graphics Applications
//  CONTENT:    main header for the VEGA Libraries
//  VERSION:    0.5.0
// ===========================================================================
//  AUTHORS:    rft     Robert F. Tobler
// ===========================================================================
//  HISTORY:
//
//  22-apr-94 09:32:11  rft     last modification
//  11-mar-94 18:49:48  rft     created
// ===========================================================================
//  MANUAL:
//
//  DESCRIPTION
//      ______________________________________________________________________
//      0. Introduction
//
//	The main vehicles of software reuse at the instiute are a number of
//	libraries that can be used by all projects developed under VEGA. These
//	libraries have been especially designed to be somewhat consistent and
//	easy to use. Any project with the intended purpose of generating part
//	of a library or a completely new library for VEGA has to conform to
//	the library conventions of VEGA.
//
//      ______________________________________________________________________
//	1. The VEGA Library Conventions
//
//	In order to make the VEGA libraries consistent a number of conventions
//	have been used throughout all modules that have been placed in the
//	libraries. This should help the user in various ways.
//
//	1.1 Default constructors
//
//	If not explicitly documented otherwise, a default constructor for each
//	class is defined and just consists of an empty body. Thus the member
//	variables are implicitly initialized to 0.
//
//      1.2 Overloaded operators
//
//      Operators have only been overloaded if the use of the operator auto-
//      matically documents its functionality. As an example, the meaning
//      of operator+ on two Vectors is pretty self-documenting, whereas
//      there is no good interpretation of operator* on two Vectors, since
//      there are three possible functionalities: dot product, cross product,
//      or even componentwise product. Therefore operator+ on two vectors is
//      defined (with the obvious functionality), but operator* on two
//      vectors is not defined and the three possible functionalities have
//      been put into named functions (dotProduct, crossProduct, cwProduct).
//
//	1.3 Explicit conversion functions
//
//	In various cases the implicit conversion function between two classes
//	has been omitted on purpose to avoid unwanted conversions based on the
//	C++ implicit conversion rules. In a lot of these cases explicit
//	conversion functions have been defined, which adhere to a common
//	naming scheme:
//
//		NewType newVariable = getNewType(variableOfOldType);
//
//	Since it may be necessary to add conversion functions to existing
//	types, it has been decided to use global functions instead of member
//	functions, which cannot be easily added.
//
//	1.4 Combined operator-assignment functions and methods
//
//	Combined operator-assignment functions and methods (operator +=,
//	operator -=, operator *=, etc.) are always defined so that following
//	equivalences are true:
//
//              [ x += y ] == [ x = x + y ]
//              [ x -= y ] == [ x = x - y ]
//              [ x *= y ] == [ x = x * y ]
//              ...
//
//      ______________________________________________________________________
//      2. The VEGA Library Structure
//
//	The VEGA libraries provide a lot of different functionality with
//	different degrees of abstraction. Therefore the modules of the library
//	have been put into the various libraries according to these criteria.
//	The following list gives a brief overview of this organization.
//
//      2.1 The VEGA Base Library
//
//              Error			error handler class
//              Tracer			tracer class
//              Test			some classes to create test programs
//              Bool			a simple type-safe boolean class
//              OutTools		tools for generating stream output
//              System			fixed length system data types
//              Tools			general and integer tool functions
//              TTools			template tools
//              Option			simple command line parser
//              Crc32			32 bit cyclic redundancy check
//
//
//      2.2 The VEGA Math Library
//
//              Math			mathematical definitions and functions
//              Crd2			double 2D coordinate and matrix class
//              Crd3			double 3D coordinate and matrix class
//              Crd4			double 4D coordinate and matrix class
//              FCrd2			float 2D coordinate and matrix class
//              FCrd3			float 3D coordinate and matrix class
//              FCrd4			float 4D coordinate and matrix class
//              SimpleFunction		function of one variable
//              SimplePolynomial	polynomial of one variable
//
//      2.3 The VEGA Graphics Library
//
//              BiTrafo         forward and backward(inverse) transformation
//              Vec2D           double 2D vectors, points and transformations
//              Vec3D           double 3D vectors, points and transformations
//              FVec2D          float 2D vectors, points and transformations
//              FVec3D          float 3D vectors, points and transformations
//              Vec             Vec2D, Vec3D, FVec2D, and FVec3D mutual
//                              conversions and projections
//
//  SEE ALSO
//      BaseLibrary,
//      MathLibrary,
//      GraphicsLibrary
//
//  AUTHORS
//      Robert F. Tobler
//      Helwig L"offelmann
// ===========================================================================
#ifndef _VEGA_H_
#define _VEGA_H_

#include <VEGA/Base/Library.h>
#include <VEGA/Math/Library.h>
#include <VEGA/Graphics/Library.h>

#endif
// ===========================================================================
